from AccessControl import getSecurityManager
from Acquisition import aq_base
from Acquisition import aq_parent
from castle.cms import cache
from castle.cms import commands
from castle.cms import utils
from castle.cms.commands import exiftool
from castle.cms.files import duplicates
from castle.cms.interfaces import ITrashed
from lxml.html import fromstring
from OFS.interfaces import IFolder
from OFS.ObjectManager import checkValidId
from plone import api
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.app.blocks.vocabularies import AvailableSiteLayouts
from plone.app.content.browser import i18n
from plone.app.layout.navigation.defaultpage import getDefaultPage
from plone.app.linkintegrity.utils import getOutgoingLinks
from plone.dexterity.interfaces import IDexterityContainer
from plone.namedfile.file import NamedBlobFile
from plone.namedfile.file import NamedBlobImage
from plone.registry import field as registry_field
from plone.registry import Record
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IUUID
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.CMFPlone import utils as ploneutils
from Products.CMFPlone.browser.syndication.adapters import SearchFeed
from Products.CMFPlone.interfaces import ISelectableConstrainTypes
from Products.CMFPlone.interfaces.syndication import IFeedItem
from Products.DCWorkflow.Expression import createExprContext
from Products.DCWorkflow.Expression import StateChangeInfo
from Products.DCWorkflow.Transitions import TRIGGER_USER_ACTION
from Products.Five import BrowserView
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.component.hooks import getSite
from zope.container.interfaces import INameChooser
from plone.app.drafts.utils import getCurrentDraft

import json
import logging
import math
import os
import shutil
import tempfile


logger = logging.getLogger('castle.cms')


def dump_object_data(obj, duplicate=False):
    try:
        state = api.content.get_state(obj=obj)
    except WorkflowException:
        state = 'published'
    base_url = obj.absolute_url()
    registry = getUtility(IRegistry)
    if obj.portal_type in registry.get('plone.types_use_view_action_in_listings', []):
        url = base_url + '/view'
    else:
        url = base_url
    return json.dumps({
        'success': True,
        'base_url': base_url,
        'url': url,
        'edit_url': base_url + '/@@edit',
        'portal_type': obj.portal_type,
        'uid': IUUID(obj),
        'workflow_state': state,
        'title': obj.Title(),
        'valid': True,
        'duplicate': duplicate
    })


class Workflow(BrowserView):

    def __call__(self):
        self.request.response.setHeader('Content-type', 'application/json')
        if self.request.form.get('action') == 'transition':
            return self.transition()

    def transition(self):
        form = self.request.form
        transition_id = form.get('transition_id')
        comment = form.get('comment', '')
        api.content.transition(obj=self.context, transition=transition_id, comment=comment)
        return json.dumps({
            'success': True
        })


class Creator(BrowserView):
    """
    Advanced content creation view.
    """
    status = ''

    def __call__(self):
        self.sm = getSecurityManager()
        self.request.response.setHeader('Content-type', 'application/json')
        self.catalog = api.portal.get_tool('portal_catalog')
        if self.request.form.get('action') == 'check':
            return self.check()
        elif self.request.form.get('action') == 'create':
            return self.create()
        elif self.request.form.get('action') == 'chunk-upload':
            return self.chunk_upload()

    def chunk_upload(self):
        chunk = int(self.request.form['chunk'])
        chunk_size = int(self.request.form['chunkSize'])
        total_size = int(self.request.form['totalSize'])
        total_chunks = int(math.ceil(float(total_size) / float(chunk_size)))
        _id = self.request.form.get('id')

        if chunk > total_chunks:
            raise Exception("More chunks than what should be possible")

        cache_key_prefix = '%s-uploads-' % '/'.join(self.context.getPhysicalPath()[1:])
        if chunk == 1:
            # initializing chunk upload
            _id = utils.get_random_string(50)
            filename = self.request.form['name']
            tmp_dir = tempfile.mkdtemp()
            tmp_filename = os.path.join(tmp_dir, filename)
            info = {
                'last_chunk': 1,
                'total_size': total_size,
                'chunk_size': chunk_size,
                'tmp_file': tmp_filename,
                'name': filename
            }
        else:
            info = cache.get(cache_key_prefix + _id)
            # check things are matching up
            if info['last_chunk'] != chunk - 1:
                raise Exception('Invalid chunk sequence')
            if info['total_size'] != total_size:
                raise Exception('Invalid total size')
            if info['chunk_size'] != chunk_size:
                raise Exception('Inconsistent chunk size')
            info['last_chunk'] = chunk

        mode = 'wb'
        if chunk > 1:
            # appending to file now
            mode = 'ab+'
            if not os.path.exists(info['tmp_file']):
                raise Exception('No tmp upload file found')
        fi = open(info['tmp_file'], mode)

        while True:
            data = self.request.form['file'].read(2 << 16)
            if not data:
                break
            fi.write(data)
        fi.close()

        if chunk == total_chunks:
            # finish upload
            dup = False
            try:
                obj = self.create_file_content(info)
            except duplicates.DuplicateException as ex:
                obj = ex.obj
                dup = True
            tmp_dir = '/'.join(info['tmp_file'].split('/')[:-1])
            shutil.rmtree(tmp_dir)
            cache.delete(cache_key_prefix + _id)
            return dump_object_data(obj, dup)
        else:
            cache.set(cache_key_prefix + _id, info)
            check_put = None
            while check_put is None:
                try:
                    check_put = cache.get(cache_key_prefix + _id)
                except:
                    cache.set(cache_key_prefix + _id, info)
        return json.dumps({
            'success': True,
            'id': _id
        })

    def detect_duplicate(self, info):
        dup_detector = duplicates.DuplicateDetector()
        md5_hash = commands.md5(info['tmp_file'])
        obj = dup_detector.get_object(md5_hash)
        if obj is not None:
            # found, use existing file...
            raise duplicates.DuplicateException(obj)
        return dup_detector, md5_hash

    def handle_auto_folder_creation(self, folder, type_):
        # we only auto publish built-in repositories, otherwise, leave it be
        try:
            if api.content.get_state(obj=folder) != 'published':
                api.content.transition(obj=folder, transition='publish')
        except WorkflowException:
            pass

        aspect = ISelectableConstrainTypes(folder, None)
        if (aspect and (
                aspect.getConstrainTypesMode() != 1 or
                [type_] != aspect.getImmediatelyAddableTypes())):
            aspect.setConstrainTypesMode(1)
            try:
                aspect.setImmediatelyAddableTypes([type_])
            except:
                pass

        if not getattr(folder, 'exclude_from_nav', False):
            # if auto generated path, exclude from nav
            folder.exclude_from_nav = True
            folder.reindexObject()

    def get_type_and_location(self, info):
        filename = info['name']
        ext = os.path.splitext(filename)[-1]
        registry = getUtility(IRegistry)
        type_ = 'File'
        location = registry.get('file_repo_location', '/file-repository')
        if ext.lower() in ['.jpg', '.jpeg', '.gif', '.png']:
            type_ = 'Image'
            location = registry.get(
                'image_repo_location', '/image-repository')
        elif ext.lower() in ['.webm', '.ogv', '.avi', '.wmv', '.m4v',
                             '.mpg', '.mpeg', '.flv', '.mp4', '.mov']:
            type_ = 'Video'
            location = registry.get(
                'video_repo_location', '/video-repository')
        elif ext.lower() in ['.mp3']:
            type_ = 'Audio'
            location = registry.get(
                'audio_repo_location', '/audio-repository')
        return type_, location

    def create_file_content(self, info):
        type_, location = self.get_type_and_location(info)
        original_location = location

        if self.request.form.get('location'):
            location = self.request.form['location']

        original_location == location
        folder = utils.recursive_create_path(self.context, location)

        md5_hash = dup_detector = None

        if original_location == location:
            # first off, check first that it wasn't already uploaded...
            # we are only checking images for now...
            if type_ == 'Image' and commands.md5:
                dup_detector, md5_hash = self.detect_duplicate(info)
            self.handle_auto_folder_creation(folder, type_)

        obj = self.create_object(folder, type_, info)
        if type_ == 'Image' and md5_hash and dup_detector:
            dup_detector.register(obj, md5_hash)
        return obj

    def create_object(self, folder, type_, info):
        filename = info['name']
        name = filename.decode("utf8")
        chooser = INameChooser(folder)
        chooser_name = name.lower().replace('aq_', '')
        newid = chooser.chooseName(chooser_name, folder.aq_parent)

        # strip metadata from file
        if type_ in ('Image', 'File') and exiftool is not None:
            try:
                exiftool(info['tmp_file'])
            except:
                logger.warn('Could not strip metadata from file: %s' % info['tmp_file'])

        fi = open(info['tmp_file'], 'r')
        try:
            # Try to determine which kind of NamedBlob we need
            # This will suffice for standard p.a.contenttypes File/Image
            # and any other custom type that would have 'File' or 'Image' in
            # its type name
            filename = ploneutils.safe_unicode(filename)
            create_opts = dict(
                type=type_,
                id=newid,
                container=folder)
            if 'Image' in type_:
                image = NamedBlobImage(data=fi, filename=filename)
                try:
                    image.focal_point = [
                        float(self.request.form.get('focalX')),
                        float(self.request.form.get('focalY'))
                    ]
                except:
                    pass
                create_opts['image'] = image
            else:
                create_opts['file'] = NamedBlobFile(data=fi, filename=filename)
            if type_ == 'Video':
                create_opts['youtube_url'] = self.request.form.get('youtube_url', '')

            if self.request.form.get('title'):
                create_opts['title'] = self.request.form.get('title')
            if self.request.form.get('description'):
                create_opts['description'] = self.request.form.get('description')
            if self.request.form.get('tags'):
                create_opts['subject'] = self.request.form.get('tags').split(';')
            return api.content.create(**create_opts)
        finally:
            fi.close()

    def get_type_id(self):
        form = self.request.form
        return form.get('selectedType[typeId]',
                        form.get('selectedType[id]')).replace('%20', ' ')

    def create(self):
        if self._check():
            path = self.request.form.get('basePath', '/')
            folder = utils.recursive_create_path(self.context, path)
            obj = api.content.create(
                type=self.get_type_id(),
                id=self.request.form['id'],
                title=self.request.form['title'],
                container=folder)
            transition_to = self.request.form.get('transitionTo')
            if transition_to:
                try:
                    api.content.transition(obj=obj, transition=transition_to)
                except:
                    pass
            return dump_object_data(obj)
        else:
            return json.dumps({
                'valid': False,
                'status': 'Unknown error creating content'
            })

    def get_folder(self):
        folder = None
        path = self.request.form.get('basePath', '/')
        if path == '/':
            folder = self.context
        else:
            path = path.lstrip('/')
            folder = self.context.restrictedTraverse(path, None)
            if folder is None:
                # check parents to see if we can auto create structure
                path = '/'.join(path.split('/')[:-1])
                while path:
                    folder = self.context.restrictedTraverse(path, None)
                    if folder:
                        if not IFolder.providedBy(folder):
                            self.status = 'Not a valid path.'
                            return None
                        break
                    path = '/'.join(path.split('/')[:-1])
                if not folder:
                    self.status = """Will automatically create folder(s) to place
content in this location."""
                    folder = self.context
            elif not IFolder.providedBy(folder):
                self.status = 'Not a valid path.'
                return None

        return folder

    def can_add(self, folder, type_id):
        constraints = ISelectableConstrainTypes(folder, None)
        addable = None
        if constraints is not None:
            addable = constraints.getLocallyAllowedTypes()

        if addable is None:
            addable = [t.getId() for t in folder.allowedContentTypes()]

        if type_id not in addable:
            return False
        return True

    def valid_id(self, container, id):
        if id in ('path', 'id', 'start'):
            return False
        try:
            checkValidId(container, id, False)
            obj = getattr(self, id, None)
            if obj is not None:
                if id not in container.objectIds():
                    return False
        except:
            return False
        return True

    def _check(self, folder=None):
        valid = True
        if folder is None:
            folder = self.get_folder()
        if folder is not None:
            portal_types = getToolByName(self.context, 'portal_types')
            type_id = self.get_type_id()
            pt = portal_types[type_id]
            add_perm = utils.get_permission_title(pt.add_permission)

            if valid and not self.can_add(folder, type_id):
                valid = False
                self.status = 'You are not allowed to add this content type here'
            elif valid and not self.sm.checkPermission(add_perm, folder):
                valid = False
                self.status = 'You do not have permission to add content here.'
            elif valid and self.request.form['id'] in folder.objectIds():
                valid = False
                ob = folder[self.request.form['id']]
                if ITrashed.providedBy(ob):
                    self.status = ('Content in recycling bin with same id exists. '
                                   'Delete, rename or restore recycled content to use this.')
                else:
                    self.status = 'Content with same ID already exists'
            elif not self.valid_id(folder, self.request.form['id']):
                valid = False
                self.status = '"%s" is not valid to be used in a path' % self.request.form['id']
        else:
            valid = False
            if not self.status:
                self.status = 'Could not validate folder'

        for idx, part in enumerate(self.request.form.get('basePath', '/').split('/')):
            if not part:
                continue
            if part in self.context:
                continue
            if not self.valid_id(self.context, part):
                valid = False
                self.status = '"%s" is not valid to be used in a path' % part
                break
        return valid, folder

    def get_workflow_info(self, folder):
        type_id = self.get_type_id()
        wf_tool = api.portal.get_tool('portal_workflow')
        cbt = wf_tool._chains_by_type
        chain = cbt.get(type_id, None)
        if chain is None:
            chain = wf_tool.getDefaultChain()

        if len(chain) == 0:
            state_info = None
        else:
            wf = wf_tool[chain[0]]
            initial_state = wf.states[wf.initial_state]
            state_info = {
                'initial': {
                    'id': initial_state.id,
                    'title': initial_state.title
                }
            }
            possible_states = {}

            for transition_id in initial_state.transitions:
                transition = wf.transitions.get(transition_id, None)
                if (transition is None or
                        transition.trigger_type != TRIGGER_USER_ACTION or
                        not transition.actbox_name):
                    continue
                try:
                    new_state = wf.states[transition.new_state_id]
                except:
                    continue
                if transition.new_state_id in possible_states:
                    # we can already get here... don't worry about it, ignore
                    continue
                checker = WorkflowPermissionChecker(initial_state, folder)
                if checker.has_permission(transition):
                    possible_states[transition.new_state_id] = {
                        'title': new_state.title,
                        'transition': transition_id
                    }
            state_info['possible_states'] = possible_states

        return state_info

    def check(self):
        valid, folder = self._check()

        if valid:
            info = self.get_workflow_info(folder)
        else:
            info = None

        return json.dumps({
            'valid': valid,
            'status': self.status,
            'stateInfo': info
        })


class WorkflowPermissionChecker(object):

    def __init__(self, initial_state, folder):
        self.folder = folder
        self.initial_state = initial_state
        self.sm = getSecurityManager()
        self.user = self.sm.getUser()
        self.folder_roles = self.user.getRolesInContext(folder)
        self._u_groups = None

    @property
    def u_groups(self):
        if self._u_groups is not None:
            return self._u_groups

        b = aq_base(self.user)
        if hasattr(b, 'getGroupsInContext'):
            u_groups = self.user.getGroupsInContext(self.folder)
        elif hasattr(b, 'getGroups'):
            u_groups = self.user.getGroups()
        else:
            u_groups = ()
        self._u_groups = u_groups
        return u_groups

    def has_permission(self, transition):
        guard = transition.guard
        if guard is None:
            return True

        for permission in guard.permissions or []:
            # need to manually check permissions here because
            # we don't have an object to check against
            check_parent = True
            if permission in self.initial_state.permissions:
                # check if acquire set for initial_state.
                # if acquire, you can check against parent
                pinfo = self.initial_state.getPermissionInfo(permission)
                if pinfo.get('acquired'):
                    check_parent = True
                else:
                    check_parent = False
                    if len(set(self.folder_roles) & set(pinfo['roles'])) > 0:
                        break
            if check_parent:
                if self.sm.checkPermission(permission, self.folder):
                    break
        else:
            # no perms valid...
            return False

        if guard.roles:
            for role in guard.roles:
                if role in self.folder_roles:
                    break
            else:
                return False

        if guard.groups:
            for group in guard.groups:
                if group in self.u_groups:
                    break
            else:
                return False

        if guard.expr is not None:
            econtext = createExprContext(
                StateChangeInfo(self.folder, aq_parent(aq_parent(self.initial_state))))
            res = guard.expr(econtext)
            if not res:
                return False

        return True


class QualityCheckContent(BrowserView):

    def __call__(self):

        valid = True
        for link in getOutgoingLinks(self.context):
            state = api.content.get_state(obj=link.to_object, default='published')
            if state != 'published':
                valid = False
                break

        headers_ordered = True

        try:
            feed = SearchFeed(api.portal.get())
            adapter = queryMultiAdapter((self.context, feed), IFeedItem)
            html = adapter.render_content_core().strip()
        except:
            html = ''
        if html:
            dom = fromstring(html)
            last = 1
            for el in dom.cssselect('h1,h2,h3,h4,h5,h6'):
                idx = int(el.tag[-1])
                if idx - last > 1:
                    # means they skipped from say h1 -> h5
                    # h1 -> h2 is allowed
                    headers_ordered = False
                    break
                last = idx

        self.request.response.setHeader('Content-type', 'application/json')
        return json.dumps({
            'title': self.context.Title(),
            'id': self.context.getId(),
            'description': self.context.Description(),
            'linksValid': valid,
            'headersOrdered': headers_ordered
        })


class PageLayoutSelector(BrowserView):

    def __call__(self):
        self.request.response.setHeader('Content-type', 'application/json')
        action = self.request.form.get('action')
        if action == 'save':
            data = self.save()
        else:
            data = self.get()
        return json.dumps(data)

    def save(self):
        adapted = ILayoutAware(self.context)
        data = self.request.form.get('data')
        if not data:
            return
        data = json.loads(data)
        adapted.pageSiteLayout = data['page_layout']
        adapted.sectionSiteLayout = data['section_layout']
        parent = aq_parent(self.context)
        if ISiteRoot.providedBy(parent):
            # check if default page...
            if getDefaultPage(parent) == self.context.id:
                # also set site wide global layout setting...
                registry = getUtility(IRegistry)
                field = registry_field.TextLine(title=u'Default layout', required=False)
                new_record = Record(field)
                registry.records['castle.cms.default_layout'] = new_record
                registry['castle.cms.default_layout'] = data['section_layout']
        return {
            'success': True
        }

    def get(self):
        available = dict([(t.value, t.title)
                          for t in AvailableSiteLayouts(self.context)])
        applied = 'index.html'
        applied_context = getSite()

        context = self.context
        adapted = context_adapted = ILayoutAware(context)
        selected = adapted.pageSiteLayout
        if selected is None:
            context = aq_parent(context)
            while not ISiteRoot.providedBy(context):
                adapted = ILayoutAware(context, None)
                if adapted and adapted.sectionSiteLayout:
                    selected = adapted.sectionSiteLayout
                    break
                context = aq_parent(context)

        if selected:
            applied = selected
            applied_context = context

        return {
            'success': True,
            'available': available,
            'applied': applied,
            'context': utils.get_path(self.context),
            'applied_context': utils.get_path(applied_context),
            'page_layout': context_adapted.pageSiteLayout,
            'section_layout': context_adapted.sectionSiteLayout,
            'folder': IDexterityContainer.providedBy(self.context)
        }


class PublishContent(BrowserView):

    def __call__(self):
        self.request.response.setHeader('Content-type', 'application/json')
        api.content.transition(self.context, 'publish')
        return json.dumps({
            'success': True
        })


class i18njs(i18n.i18njs):

    def __call__(self, domain='widgets', language='en'):
        return super(i18njs, self).__call__(domain, language)


class PingCurrentDraft(BrowserView):
    """
    To add to the current draft modified time
    """

    def __call__(self):
        draft = getCurrentDraft(self.request, create=False)
        if draft is not None:
            draft._p_changed = 1



from castle.cms.media import video
from castle.cms.utils import retriable
from collective.celery import task


@retriable()
def _process_video(obj):
    video.process(obj)


@task()
def process_video(obj):
    _process_video(obj)


from AccessControl.SecurityManagement import newSecurityManager
from collective.elasticsearch.es import ElasticSearchCatalog
from collective.elasticsearch.hook import index_batch
from plone import api
from plone.uuid.interfaces import IUUID
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from tendo import singleton
from zope.component.hooks import setSite

import logging
import transaction


logger = logging.getLogger('castle.cms')


def index_site(site):
    setSite(site)
    catalog = api.portal.get_tool('portal_catalog')
    es = ElasticSearchCatalog(catalog)
    if not es.enabled:
        return

    # first we want to get all document ids from elastic
    page_size = 700
    ids = []
    result = es.connection.search(
        index=es.index_name, doc_type=es.doc_type,
        scroll='30s',
        size=page_size,
        fields=[],
        body={
            "query": {
                "match_all": {}
            }
        })
    ids.extend([r['_id'] for r in result['hits']['hits']])
    scroll_id = result['_scroll_id']
    while scroll_id:
        result = es.connection.scroll(
            scroll_id=scroll_id,
            scroll='30s'
        )
        if len(result['hits']['hits']) == 0:
            break
        ids.extend([r['_id'] for r in result['hits']['hits']])
        scroll_id = result['_scroll_id']

    index = {}
    count = 0
    for brain in catalog():
        count += 1
        # go through each object and reindex using bulk setting
        try:
            ob = brain.getObject()
        except:
            print('Could not get object of %s' % brain.getPath())
            continue
        try:
            uid = IUUID(ob)
            index[uid] = ob
        except TypeError:
            print('Could not get UID of %s' % brain.getPath())
            continue
        if uid in ids:
            # remove from uids... When all said and done,
            # we'll make sure the uids left are in fact no longer on the
            # system and remove them from es
            ids.remove(uid)
        if len(index) > 300:
            print('finished indexing %i' % count)
            index_batch([], index, [], es)
            site._p_jar.invalidateCache()  # noqa
            transaction.begin()
            site._p_jar.sync()  # noqa
            index = {}
    index_batch([], index, [], es)

    remove = []
    for uid in ids:
        brains = catalog(UID=uid)
        if len(brains) == 0:
            remove.append(uid)
    index_batch(remove, {}, [], es)


def run(app):
    singleton.SingleInstance('reindexes')

    user = app.acl_users.getUser('admin')  # noqa
    newSecurityManager(None, user.__of__(app.acl_users))  # noqa

    for oid in app.objectIds():  # noqa
        obj = app[oid]  # noqa
        if IPloneSiteRoot.providedBy(obj):
            index_site(obj)


if __name__ == '__main__':
    run(app)  # noqa



from AccessControl.SecurityManagement import newSecurityManager
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from tendo import singleton
from zope.component.hooks import setSite
from plone import api

import logging
import transaction
import time


logger = logging.getLogger('castle.cms')


DRAFT_DURATION = 60 * 60 * 1  # 1 hour


def clean(site):
    setSite(site)
    pdrafts = api.portal.get_tool('portal_drafts')
    for userid in pdrafts.drafts.keys():
        draft_groups = pdrafts.drafts[userid]
        for uid in draft_groups.keys():
            drafts = draft_groups[uid]
            for draft_name in drafts.keys():
                draft = drafts[draft_name]
                if (time.time() - draft._p_mtime) >= DRAFT_DURATION:
                    logger.info('cleaning draft {}'.format(uid))
                    pdrafts.discardDraft(draft)
    transaction.commit()


def run(app):
    singleton.SingleInstance('cleanusers')

    user = app.acl_users.getUser('admin')  # noqa
    newSecurityManager(None, user.__of__(app.acl_users))  # noqa

    for oid in app.objectIds():  # noqa
        obj = app[oid]  # noqa
        if IPloneSiteRoot.providedBy(obj):
            try:
                clean(obj)
            except:
                logger.error('Could not clean users %s' % oid, exc_info=True)


if __name__ == '__main__':
    run(app)  # noqa



from plone.folder.default import DefaultOrdering


class ReversedOrdering(DefaultOrdering):
    """return reversed ids."""

    def notifyAdded(self, obj_id):
        """ see interfaces.py """
        order = self._order(True)
        order.insert(0, obj_id)

    def notifyRemoved(self, obj_id):
        """ see interfaces.py """
        order = self._order()
        idx = order.index(obj_id)
        del order[idx]

    def getObjectPosition(self, obj_id):
        """ see interfaces.py """
        order = self._order()
        if obj_id in order:
            return order.index(obj_id)
        raise ValueError('No object with id "{0:s}" exists.'.format(obj_id))

    def _pos(self, create=False):
        # XXX bbb here so super class has something to use
        # I don't understand why we store the order separate
        # from the list of ids when the list of ids is also ordered
        return {}



from castle.cms.cron.utils import login_as_admin
from castle.cms.cron.utils import setup_site
from plone.app.blocks import tiles
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.app.blocks.utils import getLayout
from plone.app.uuid.utils import uuidToCatalogBrain
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from repoze.xmliter.utils import getHTMLSerializer
from unidecode import unidecode
from zope.globalrequest import getRequest

import logging
import requests


logger = logging.getLogger('castle.cms')


class BadResponse(object):
    status_code = 500


_safe = [
    'https://twitter.com/intent/tweet',
    'https://www.facebook.com/sharer/sharer.php',
    'http://www.addthis.com',
    'javascript:'
]

def find_url(ob, url):
    if 'tmbc.me' in url:
        return False

    for safe in _safe:
        if safe in url:
            return True

    if 'resolveuid/' in url:
        # check if object...
        uid = url.split('resolveuid/')[-1]
        uid = uid.split('/')[0]
        return uuidToCatalogBrain(uid) is not None
    elif ('https://' in url or 'http://' in url) and 'http://nohost' not in url:
        try:
            print('checking ' + url)
            resp = requests.get(url)
        except:
            resp = BadResponse()
        return resp.status_code == 200
    else:
        if 'http://nohost' in url:
            url = url.replace('http://nohost', '')
        url = url.replace('%20', ' ').split('?')[0].split('#')[0].split('/@@images')[0]
        try:
            return ob.restrictedTraverse(str(url), None) is not None
        except:
            return False


def find_broken(site):
    setup_site(site)
    catalog = site.portal_catalog

    broken = []
    good_urls = []

    req = getRequest()
    for brain in catalog(object_provides=ILayoutAware.__identifier__):
        ob = brain.getObject()
        layout = getLayout(ob)
        dom = getHTMLSerializer(layout)
        tiles.renderTiles(req, dom.tree, ob.absolute_url() + '/layout_view')
        root = dom.tree.getroot()
        for anchor in root.cssselect('a'):
            if not anchor.attrib.get('href'):
                continue
            url = anchor.attrib['href']
            if url[0] == '#' or url.startswith('data:') or url.startswith('mailto:'):
                continue
            if url in good_urls:
                continue

            if find_url(ob, url):
                good_urls.append(url)
            else:
                try:
                    text = unidecode(anchor.text_content())
                except:
                    text = ''
                result = '{} linking to broken -> {}({})'.format(
                    brain.getPath(),
                    url,
                    text
                )
                broken.append(result)
                print(result)

        for img in root.cssselect('img'):
            if not img.attrib.get('src'):
                continue
            url = img.attrib['src']
            if url[0] == '#' or url.startswith('data:'):
                continue
            if find_url(ob, url):
                good_urls.append(url)
            else:
                result = '{} linking to broken image -> {}'.format(
                    brain.getPath(),
                    url
                )
                broken.append(result)
                print(result)

    filename = 'broken-links-{}.txt'
    fi = open(filename, 'w')
    fi.write('\n'.join(broken))
    fi.close()


if __name__ == '__main__':
    login_as_admin(app)  # noqa

    for oid in app.objectIds():  # noqa
        obj = app[oid]  # noqa
        if IPloneSiteRoot.providedBy(obj):
            try:
                find_broken(obj)
            except:
                logger.error('Could not clean users %s' % oid, exc_info=True)



import time

import requests
from plone import api
from plone.protect.interfaces import IDisableCSRFProtection
from plone.uuid.interfaces import IUUID
from Products.Five import BrowserView
from zope.interface import alsoProvides
from zope.security import checkPermission


class API(object):

    def __init__(self):
        self.url = api.portal.get_registry_record('castle.etherpad_url').rstrip('/')
        self.key = api.portal.get_registry_record('castle.etherpad_api_key')

    @property
    def valid(self):
        return self.url and self.key

    def __call__(self, method, **kwargs):
        url = self.url + '/api/1/' + method
        kwargs['apikey'] = self.key
        resp = requests.get(url, params=kwargs)
        return resp.json()


class PadView(BrowserView):
    contribute_url = None
    text = None

    def get_iframe_url(self, eapi):
        """
        things to do here:
            1. make sure user is created
            2. create group for pad. One group for every pad
            3. create pad for group
            4. create session for user
        """
        user = api.user.get_current()
        user_id = getattr(user, '_etherpad_id', None)
        writing = False
        if user_id is None:
            result = eapi('createAuthorIfNotExistsFor',
                          authorMapper=user.getId(),
                          name=user.getProperty('fullname') or user.getId())
            user._etherpad_id = user_id = result['data']['authorID']
            writing = True

        group_id = getattr(self.context, '_etherpad_group_id', None)
        if group_id is None:
            pad_uid = IUUID(self.context)
            result = eapi('createGroupIfNotExistsFor',
                          groupMapper=pad_uid)

            self.context._etherpad_group_id = group_id = result['data']['groupID']
            writing = True

        pad_id = getattr(self.context, '_etherpad_pad_id', None)
        if pad_id is None:
            result = eapi('createGroupPad',
                          groupID=group_id,
                          padName=self.context.title)

            if result.get('code') == 1 and result['data'] is None:
                result = eapi('listPads',
                              groupID=group_id)
                self.context._etherpad_pad_id = pad_id = result['data']['padIDs'][0]
            else:
                self.context._etherpad_pad_id = pad_id = result['data']['padID']
            writing = True

        if writing:
            alsoProvides(self.request, IDisableCSRFProtection)

        result = eapi('createSession',
                      groupID=group_id,
                      authorID=user_id,
                      validUntil=str(int(time.time() + 24 * 60 * 60)))

        session_id = result['data']['sessionID']
        self.request.response.setCookie(
            'sessionID',
            session_id,
            quoted=False,
            path='/'
        )
        return '%s/p/%s' % (
            eapi.url,
            pad_id
        )

    def get_text(self, eapi):
        pad_id = getattr(self.context, '_etherpad_pad_id', None)
        if pad_id is None:
            return 'No pad found to render'
        result = eapi('getHTML', padID=pad_id)
        if result['code'] == 0:
            return result['data']['html']
        else:
            return 'Pad text can not be found'

    def __call__(self):
        eapi = API()

        if eapi.valid:
            self.valid = True
            if checkPermission('cmf.ModifyPortalContent', self.context):
                self.contribute_url = self.get_iframe_url(eapi)
            else:
                self.text = self.get_text(eapi)
        else:
            self.valid = False

        return self.index()



from collective.documentviewer.convert import DUMP_FILENAME
from tempfile import mkdtemp
from logging import getLogger
import os
import subprocess
import shutil


TMP_PDF_FILENAME = 'dump.pdf'

logger = getLogger('wildcard.media')


class BaseSubProcess(object):
    default_paths = ['/bin', '/usr/bin', '/usr/local/bin']
    bin_name = ''

    if os.name == 'nt':
        close_fds = False
    else:
        close_fds = True

    def __init__(self):
        binary = self._findbinary()
        self.binary = binary
        if binary is None:
            raise IOError("Unable to find %s binary" % self.bin_name)

    def _findbinary(self):
        if 'PATH' in os.environ:
            path = os.environ['PATH']
            path = path.split(os.pathsep)
        else:
            path = self.default_paths

        for directory in path:
            fullname = os.path.join(directory, self.bin_name)
            if os.path.exists(fullname):
                return fullname

        return None

    def _run_command(self, cmd, or_error=False):
        if isinstance(cmd, basestring):
            cmd = cmd.split()
        cmdformatted = ' '.join(cmd)
        logger.info("Running command %s" % cmdformatted)
        process = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE,
                                   close_fds=self.close_fds)
        output, error = process.communicate()
        process.stdout.close()
        process.stderr.close()
        if process.returncode != 0:
            error = u"""Command
%s
finished with return code
%i
and output:
%s
%s""" % (cmdformatted, process.returncode, output, error.decode('utf8'))
            logger.info(error)
            raise Exception(error)
        logger.info("Finished Running Command %s" % cmdformatted)
        if not output:
            if or_error:
                return error
        return output


class AVConvProcess(BaseSubProcess):
    """
    """
    if os.name == 'nt':
        bin_name = 'avconv.exe'
    else:
        bin_name = 'avconv'

    def grab_frame(self, filepath, outputfilepath, instant='00:00:5'):
        cmd = [self.binary, '-i', filepath, '-ss', instant, '-f', 'image2',
               '-vframes', '1', outputfilepath]
        self._run_command(cmd)

    def convert(self, filepath, outputfilepath):
        ext = filepath.split('.')[-1]
        if ext.lower() == 'wmv':
            cmd = [
                self.binary, '-i', filepath,
                '-map_metadata', '-1',
                '-vcodec', 'libx264', '-preset', 'ultrafast', '-profile:v',
                'baseline', '-acodec', 'aac', '-strict', 'experimental',
                '-r', '24', '-b', '255k', '-ar', '44100', '-ab', '59k',
                outputfilepath]
        if ext.lower() == 'mov':
            # files that are mov are problematic...
            # need to first converted to ogv and then back to mp4?
            # stupid, but what can you do?
            tmpoutput_path = outputfilepath.replace('.mp4', '.ogv')
            cmd = [self.binary, '-i', filepath, tmpoutput_path]
            self._run_command(cmd)
            filepath = tmpoutput_path
        if ext != 'wmv':
            cmd = [
                self.binary, '-i', filepath,
                '-vcodec', 'h264', '-acodec', 'aac', '-strict', '-2',
                outputfilepath]
        self._run_command(cmd)


try:
    avconv = AVConvProcess()
except IOError:
    avconv = None
    logger.warn('ffmpeg not installed. castle.cms will not be able to convert video')


class ExifToolProcess(BaseSubProcess):
    """
    """
    if os.name == 'nt':
        bin_name = 'exiftool.exe'
    else:
        bin_name = 'exiftool'

    def __call__(self, filepath):
        cmd = [self.binary, '-all=', filepath]
        self._run_command(cmd)

try:
    exiftool = ExifToolProcess()
except IOError:
    exiftool = None
    logger.warn('exiftool not installed. castle.cms will not be able to strip metadata')


class MD5SubProcess(BaseSubProcess):
    """
    To get md5 hash of files on the filesystem so
    large files do not need to be loaded into
    memory to be checked
    """
    if os.name == 'nt':
        bin_name = 'md5.exe'
    else:
        bin_name = 'md5'

    def __call__(self, filepath):
        cmd = [self.binary, filepath]
        hashval = self._run_command(cmd)
        return hashval.split('=')[1].strip()

try:
    md5 = MD5SubProcess()
except IOError:
    md5 = None


class MD5SumSubProcess(BaseSubProcess):
    """
    To get md5 hash of files on the filesystem so
    large files do not need to be loaded into
    memory to be checked
    """
    if os.name == 'nt':
        bin_name = 'md5sum.exe'
    else:
        bin_name = 'md5sum'

    def __call__(self, filepath):
        cmd = [self.binary, filepath]
        hashval = self._run_command(cmd)
        return hashval.split('  ')[0].strip()

try:
    if md5 is None:
        md5 = MD5SumSubProcess()
except IOError:
    logger.exception("No md5sum or md5 installed. castle.cms "
                     "will not be able to detect md5 of files.")
    md5 = None


class DocSplitSubProcess(BaseSubProcess):
    """
    idea of how to handle this shamelessly
    stolen from ploneformgen's gpg calls
    """

    if os.name == 'nt':
        bin_name = 'docsplit.exe'
    else:
        bin_name = 'docsplit'

    def dump_image(self, data, size, _format):
        # docsplit images pdf.pdf --size 700x,300x,50x
        # --format gif --output
        tmpdir = mkdtemp()
        tmpfilepath = os.path.join(tmpdir, TMP_PDF_FILENAME)
        tmpfi = open(tmpfilepath, 'wb')
        tmpfi.write(data)
        tmpfi.close()
        cmd = [
            self.binary, "images", tmpfilepath,
            '--language', 'eng',
            '--size', size,
            '--format', _format,
            '--output', tmpdir,
            '--pages', '1']

        self._run_command(cmd)
        os.remove(tmpfilepath)
        return os.path.join(tmpdir, 'dump_1.gif')

    def convert_to_pdf(self, filepath, filename, output_dir):
        # get ext from filename
        ext = os.path.splitext(os.path.normcase(filename))[1][1:]
        inputfilepath = os.path.join(output_dir, 'dump.%s' % ext)
        shutil.move(filepath, inputfilepath)
        orig_files = set(os.listdir(output_dir))
        cmd = [
            self.binary, 'pdf', inputfilepath,
            '--output', output_dir]
        self._run_command(cmd)

        # remove original
        os.remove(inputfilepath)

        # while using libreoffice, docsplit leaves a 'libreoffice'
        # folder next to the generated PDF, removes it!
        libreOfficePath = os.path.join(output_dir, 'libreoffice')
        if os.path.exists(libreOfficePath):
            shutil.rmtree(libreOfficePath)

        # move the file to the right location now
        files = set(os.listdir(output_dir))

        if len(files) != len(orig_files):
            # we should have the same number of files as when we first began
            # since we removed libreoffice.
            # We do this in order to keep track of the files being created
            # and used...
            raise Exception("Error converting to pdf")

        converted_path = os.path.join(output_dir,
                                      [f for f in files - orig_files][0])
        shutil.move(converted_path, os.path.join(output_dir, DUMP_FILENAME))


try:
    docsplit = DocSplitSubProcess()
except IOError:
    logger.exception("No docsplit installed.")
    docsplit = None



from plone import api
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.resources.browser.cook import cookWhenChangingSettings


PROFILE_ID = 'profile-castle.cms.upgrades:2_0_12'


def upgrade(context, logger=None):
    setup = getToolByName(context, 'portal_setup')
    setup.runAllImportStepsFromProfile(PROFILE_ID)
    cookWhenChangingSettings(api.portal.get())



from AccessControl import Unauthorized
from Acquisition import aq_parent
from castle.cms import cache
from castle.cms import linkintegrity
from castle.cms import utils
from castle.cms.utils import retriable
from collective.celery import task
from plone import api
from Products.CMFPlone.utils import pretty_title_or_id

import logging
import transaction


logger = logging.getLogger('castle.cms')


def paste_error_handle(where, op, mdatas):
    user = api.user.get_current()
    email = user.getProperty('email')
    if email:
        name = user.getProperty('fullname') or user.getId()
        try:
            utils.send_email(
                recipients=email,
                subject="Paste Operation Failed(Site: %s)" % (
                    api.portal.get_registry_record('plone.site_title')),
                html="""
<p>Hi %s,</p>

<p>The site has failed to paste your items into the /%s folder.</p>

<p>Please contact your administrator.</p>""" % (
                    name, where.lstrip('/')))
        except:
            logger.warn('Could not send status email ', exc_info=True)


@retriable(on_retry_exhausted=paste_error_handle)
def _paste_items(where, op, mdatas):
    logger.info('Copying a bunch of items')
    portal = api.portal.get()
    catalog = api.portal.get_tool('portal_catalog')
    dest = portal.restrictedTraverse(str(where.lstrip('/')))

    count = 0
    commit_count = 0
    portal._p_jar.sync()

    try:
        if mdatas[0][0].startswith('cache:'):
            cache_key = mdatas[0][0].replace('cache:', '')
            mdatas = cache.get(cache_key)
    except IndexError:
        pass

    for mdata in mdatas[:]:
        count += len(catalog(path={'query': '/'.join(mdata), 'depth': -1}))
        ob = portal.unrestrictedTraverse(str('/'.join(mdata)), None)
        if ob is None:
            continue
        if op == 0:
            # copy
            api.content.copy(ob, dest, safe_id=True)
        else:
            api.content.move(ob, dest, safe_id=True)

        if count / 50 != commit_count:
            # commit every 50 objects moved
            transaction.commit()
            commit_count = count / 50
            portal._p_jar.sync()
            # so we do not redo it
            try:
                mdatas.remove(mdata)
            except:
                pass

    # we commit here so we can trigger conflict errors before
    # trying to send email
    transaction.commit()

    user = api.user.get_current()
    email = user.getProperty('email')
    if email:
        name = user.getProperty('fullname') or user.getId()
        try:
            utils.send_email(
                recipients=email,
                subject="Paste Operation Finished(Site: %s)" % (
                    api.portal.get_registry_record('plone.site_title')),
                html="""
    <p>Hi %s,</p>

    <p>The site has finished pasting items into /%s folder.</p>""" % (
                    name, where.lstrip('/')))
        except:
            logger.warn('Could not send status email ', exc_info=True)


@task()
def paste_items(where, op, mdatas):
    _paste_items(where, op, mdatas)


def delete_error_handle(where, op, mdatas):
    user = api.user.get_current()
    email = user.getProperty('email')
    if email:
        name = user.getProperty('fullname') or user.getId()
        try:
            utils.send_email(
                recipients=email,
                subject="Delete Operation Failed(Site: %s)" % (
                    api.portal.get_registry_record('plone.site_title')),
                html="""
<p>Hi %s,</p>

<p>The site has failed to delete the items.</p>

<p>Please contact your administrator.</p>""" % (
                    name, where.lstrip('/')))
        except:
            logger.warn('Could not send status email ', exc_info=True)


@retriable(on_retry_exhausted=delete_error_handle)
def _delete_items(uids):
    user = api.user.get_current()
    email = user.getProperty('email')
    name = user.getProperty('fullname') or user.getId()
    errors = []
    deleted = []
    site_path = '/'.join(api.portal.get().getPhysicalPath())

    for brain in api.portal.get_tool('portal_catalog')(UID=uids):
        obj = brain.getObject()

        try:
            lock_info = obj.restrictedTraverse('@@plone_lock_info')
        except AttributeError:
            lock_info = None
        if lock_info is not None and lock_info.is_locked():
            try:
                plone_lock_operations = obj.restrictedTraverse('@@plone_lock_operations')
                plone_lock_operations.safe_unlock()
            except AttributeError:
                pass

        parent = aq_parent(obj)
        obj_path = '/'.join(obj.getPhysicalPath())[len(site_path):]
        title = pretty_title_or_id(obj, obj)
        try:
            parent.manage_delObjects(obj.getId())
            deleted.append({
                'path': obj_path,
                'title': title})
        except Unauthorized:
            errors.append({
                'path': obj_path,
                'msg': 'Unauthorized to delete object',
                'title': title
            })
        except:
            errors.append({
                'path': obj_path,
                'msg': 'Unknown error attempting to delete',
                'title': title
            })

    # we commit here so we can trigger conflict errors before
    # trying to send email
    transaction.commit()

    if email:
        errors_html = ''
        if len(errors) > 0:
            error_items_html = ''
            for error in errors:
                error_items_html += '<li>{0}({1}): {2}</li>'.format(
                    error['title'], error['path'], error['msg'])
            errors_html = """
<h3>There were errors attempting to delete some of the items</h3>
<ul>%s</ul>""" % error_items_html

        items_html = ''
        for item in deleted:
            items_html += '<li>{0}({1})</li>'.format(item['title'], item['path'])
        deleted_html = """
<h3>Deleted items</h3>
<ul>{0}</ul>""".format(items_html)
        try:
            utils.send_email(
                recipients=email,
                subject="Finished deleting items(Site: %s)" % (
                    api.portal.get_registry_record('plone.site_title')),
                html="""
<p>Hi {0},</p>

<p>The site has finished deleting items you asked for.</p>

{1}

{2}
""".format(name, deleted_html, errors_html))
        except:
            logger.warn('Could not send status email ', exc_info=True)

@task()
def delete_items(uids):

    _delete_items(uids)


@task()
def scan_links(obj):
    if isinstance(obj, basestring):
        obj = api.portal.get().restrictedTraverse(obj, None)
        if obj is None:
            return
    linkintegrity.scan(obj)


@task.as_admin()
def trash_tree(obj):
    _trash_tree(obj)


@retriable()
def _trash_tree(obj):
    """
    this is for restoring and trashing content.
    goes up the tree and re-indexes
    """
    catalog = api.portal.get_tool('portal_catalog')

    # first, query with objects that are trashed
    for brain in catalog(path={'query': '/'.join(obj.getPhysicalPath()),
                               'depth': -1},
                         trashed=True):
        ob = brain.getObject()
        # we just want to reindex because trashed should get picked up for indexing now
        ob.reindexObject(idxs=['trashed', 'modified'])

    # then for ones that are not
    for brain in catalog(path={'query': '/'.join(obj.getPhysicalPath()),
                               'depth': -1}):
        ob = brain.getObject()
        # we just want to reindex because trashed should get picked up for indexing now
        ob.reindexObject(idxs=['trashed', 'modified'])



from Acquisition import aq_inner
from castle.cms.lockout import LockoutManager
from plone import api
from Products.CMFPlone.controlpanel.browser import usergroups_usersoverview
from zExceptions import Forbidden

import time


class UsersOverviewControlPanel(usergroups_usersoverview.UsersOverviewControlPanel):
    def manageUser(self, users=[], resetpassword=[], delete=[]):
        super(UsersOverviewControlPanel, self).manageUser(
            users=users, resetpassword=resetpassword, delete=delete)
        # custom actions here...
        action = self.request.form.get('action')
        if action == 'disable':
            self.disable_user()
        if action == 'enable':
            self.enable_user()
        elif action == 'delete':
            self.delete_user()
        elif action == 'setpassword':
            self.set_password()
        elif action == 'sendpwreset':
            self.send_pwreset()
        elif action == 'resetattempts':
            self.clear_login_attempts()

    def disable_user(self):
        # disabling user is just unassigning all roles...
        userid = self.request.form.get('userid')

        mtool = api.portal.get_tool('portal_membership')
        acl_users = api.portal.get_tool('acl_users')

        member = mtool.getMemberById(userid)
        acl_users.userFolderEditUser(
            userid, None, [], member.getDomains(), REQUEST=self.context.REQUEST)

    def enable_user(self):
        # disabling user is just unassigning all roles...
        userid = self.request.form.get('userid')

        mtool = api.portal.get_tool('portal_membership')
        acl_users = api.portal.get_tool('acl_users')

        member = mtool.getMemberById(userid)
        acl_users.userFolderEditUser(
            userid, None, ['Member'], member.getDomains(), REQUEST=self.context.REQUEST)

    def clear_login_attempts(self):
        userid = self.request.form.get('userid')
        user = api.user.get(userid)
        lockout = LockoutManager(self.context, userid)
        lockout.clear()
        lockout = LockoutManager(self.context, user.getUserName())
        lockout.clear()

    def delete_user(self):
        """
        a modified version that does not go through all content
        and reindex
        """

        userid = self.request.form.get('userid')
        # this method exists to bypass the 'Manage Users' permission check
        # in the CMF member tool's version
        context = aq_inner(self.context)

        mtool = api.portal.get_tool('portal_membership')
        acl_users = api.portal.get_tool('acl_users')

        # Delete members in acl_users.
        acl_users = context.acl_users
        member = mtool.getMemberById(userid)
        if not member.canDelete():
            raise Forbidden
        if 'Manager' in member.getRoles() and not self.is_zope_manager:
            raise Forbidden
        try:
            acl_users.userFolderDelUsers([userid])
        except (AttributeError, NotImplementedError):
            raise NotImplementedError('The underlying User Folder '
                                      'doesn\'t support deleting members.')

        # Delete member data in portal_memberdata.
        mdtool = api.portal.get_tool('portal_memberdata')
        if mdtool is not None:
            mdtool.deleteMemberData(userid)

    def initialize_memberdata(self):
        portal_memberdata = api.portal.get_tool("portal_memberdata")
        if not portal_memberdata.hasProperty("reset_password_required"):
            portal_memberdata.manage_addProperty(
                id="reset_password_required", value=False, type="boolean")
            portal_memberdata.manage_addProperty(
                id="reset_password_time", value=time.time(), type="float")

    def set_password(self):
        self.initialize_memberdata()
        userid = self.request.form.get('userid')
        pw = self.request.form.get('password')

        mtool = api.portal.get_tool('portal_membership')

        # marker to tell us we need to force user to reset password later
        user = api.user.get(userid)
        user.setMemberProperties(mapping={
            'reset_password_required': True,
            'reset_password_time': time.time()
        })

        member = mtool.getMemberById(userid)
        member.setSecurityProfile(password=pw)

    def send_pwreset(self):
        userid = self.request.form.get('userid')

        mtool = api.portal.get_tool('portal_membership')
        regtool = api.portal.get_tool('portal_registration')
        acl_users = api.portal.get_tool('acl_users')

        member = mtool.getMemberById(userid)
        current_roles = member.getRoles()

        if 'Manager' in current_roles and not self.is_zope_manager:
            raise Forbidden

        pw = regtool.generatePassword()
        acl_users.userFolderEditUser(
            userid, pw, current_roles, member.getDomains(), REQUEST=self.context.REQUEST)

        self.context.REQUEST.form['new_password'] = pw
        regtool.mailPassword(userid, self.context.REQUEST)



from zope import schema
from castle.cms.tiles.base import BaseTile
from plone.supermodel import model


class PinTile(BaseTile):
    def render(self):
        if 'PIN_TILE' not in self.request.environ:
            self.renderJS = True
            self.request.environ['PIN_TILE'] = True
        else:
            self.renderJS = False

        return self.index()


class IPinTileSchema(model.Schema):

    url = schema.URI(
        title=u"URL of Pinterest Pin to embed.",
        required=True
    )



# from plone.app.z3cform.widget import AjaxSelectFieldWidget
# from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from zope import schema
from zope.interface import alsoProvides


class ISearch(model.Schema):

    model.fieldset(
        'settings',
        fields=['searchterm_pins'],
    )

    searchterm_pins = schema.List(
        title=u'Search Term Pins',
        description=u'Searches this content should show top in results for.',
        required=False,
        value_type=schema.TextLine(),
        default=[]
    )

    # form.widget(
    #     'searchterm_pins',
    #     AjaxSelectFieldWidget
    # )


alsoProvides(ISearch, IFormFieldProvider)


# -*- coding: utf-8 -*-
from castle.cms import constants
from castle.cms import shield
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import TEST_USER_NAME
from plone.registry.interfaces import IRegistry
from zExceptions import Redirect
from zope.component import queryUtility

import unittest


SHIELD = constants.SHIELD


class TestTwoFactor(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    def test_raises_redirect_exception_when_shield_active(self):
        logout()
        registry = queryUtility(IRegistry)
        registry['plone.login_shield_setting'] = SHIELD.ALL
        self.request.PARENTS = [self.portal]
        self.assertRaises(Redirect, shield.protect, self.request)

    def test_not_raises_redirect_exception_when_shield_not_active(self):
        logout()
        registry = queryUtility(IRegistry)
        registry['plone.login_shield_setting'] = SHIELD.NONE
        self.request.PARENTS = [self.portal]
        shield.protect(self.request)

    def test_not_raises_redirect_exception_when_shield_active_and_logged_in(self):
        login(self.portal, TEST_USER_NAME)
        registry = queryUtility(IRegistry)
        registry['plone.login_shield_setting'] = SHIELD.ALL
        self.request.PARENTS = [self.portal]
        shield.protect(self.request)



import os
import argparse
import time
import hashlib

parser = argparse.ArgumentParser(description='')
parser.add_argument('--command', dest='command')
parser.add_argument('--dirs', dest='dirs', nargs='+')

args = parser.parse_args()

times = {}


def md5(fname):
    hash_md5 = hashlib.md5()
    with open(fname, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            hash_md5.update(chunk)
    return hash_md5.hexdigest()


def check(directory):
    changed = False
    for filename in os.listdir(directory):
        filepath = os.path.join(directory, filename)

        if os.path.isdir(filepath):
            if check(filepath):
                changed = True
        else:
            hashed = md5(filepath)
            current_hash = times.get(filepath, None)

            times[filepath] = hashed

            if hashed != current_hash:
                print(filepath + ' has changed!')
                changed = True

    return changed


def check_all(directories):
    changed = False
    for directory in directories:
        if check(directory):
            changed = True

    return changed


# initialize values...
def initialize(directory):
    for filename in os.listdir(directory):
        filepath = os.path.join(directory, filename)

        if os.path.isdir(filepath):
            initialize(filepath)
        else:
            changed_time = os.stat(filepath).st_mtime
            times[filepath] = changed_time
for directory in args.dirs:
    initialize(directory)


os.system(args.command)

while True:
    if check_all(args.dirs):
        # now run command...
        os.system(args.command)
    time.sleep(0.5)



from castle.cms import install
from cStringIO import StringIO
from plone.app.robotframework.testing import PloneRobotFixture
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import MOCK_MAILHOST_FIXTURE
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2
from Products.CMFPlone.resources.browser.combine import combine_bundles
from Products.CMFPlone.testing import PRODUCTS_CMFPLONE_ROBOT_REMOTE_LIBRARY_FIXTURE  # noqa
from selenium.webdriver.firefox.firefox_binary import FirefoxBinary
from zope.configuration import xmlconfig
from zope.globalrequest import clearRequest
from zope.globalrequest import setRequest
from ZPublisher import HTTPResponse

import base64
import os
import re
import sys
import transaction
import unittest


class CastleLayer(PloneSandboxLayer):
    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Fix subrequest not fallbacking to wrong encoding in test environment:
        HTTPResponse.default_encoding = 'utf-8'

        import plone.app.contenttypes
        self.loadZCML(package=plone.app.contenttypes)

        import castle.cms
        import castle.cms.tests
        xmlconfig.file('configure.zcml',
                       castle.cms,
                       context=configurationContext)
        xmlconfig.file('configure.zcml',
                       castle.cms.tests,
                       context=configurationContext)

    def setUpPloneSite(self, portal):
        # Configure five.globalrequest
        portal.REQUEST['PARENTS'] = [portal.__parent__]
        setRequest(portal.REQUEST)

        # Install into Plone site using portal_setup
        applyProfile(portal, 'plone.app.contenttypes:default')
        applyProfile(portal, 'plone.app.mosaic:default')
        applyProfile(portal, 'castle.cms:default')
        portal.portal_workflow.setDefaultChain('simple_publication_workflow')

        install.tiles(portal, portal.REQUEST)
        combine_bundles(portal)

        # Clear globalrequest
        clearRequest()


class SeleniumCastleLayer(CastleLayer):
    defaultBases = (z2.ZSERVER_FIXTURE, PLONE_FIXTURE)

    def testSetUp(self):
        # Start up Selenium
        driver = os.environ.get('SELENIUM_DRIVER', '').lower() or 'firefox'
        webdriver = __import__(
            'selenium.webdriver.%s.webdriver' % driver, fromlist=['WebDriver'])
        args = [arg.strip() for arg in
                os.environ.get('SELENIUM_ARGS', '').split()
                if arg.strip()]
        kwargs = {}
        if 'FIREFOX_BINARY' in os.environ:
            log_fi = open('/opt/plone/selenium.log', 'w')
            kwargs['firefox_binary'] = FirefoxBinary(
                os.environ['FIREFOX_BINARY'],
                log_file=log_fi)
        self['selenium'] = webdriver.WebDriver(*args, **kwargs)

    def testTearDown(self):
        self['selenium'].quit()
        del self['selenium']


class CastleRobotLayer(PloneRobotFixture):
    pass


CASTLE_FIXTURE = CastleLayer()
CASTLE_PLONE_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(CASTLE_FIXTURE, MOCK_MAILHOST_FIXTURE),
    name="CastleTesting:Functional")
CASTLE_PLONE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(CASTLE_FIXTURE, MOCK_MAILHOST_FIXTURE),
    name="CastleTesting:Integration")

CASTLE_ROBOT_FIXTURE = CastleRobotLayer()
CASTLE_ROBOT_TESTING = FunctionalTesting(
    bases=(CASTLE_ROBOT_FIXTURE, CASTLE_FIXTURE, MOCK_MAILHOST_FIXTURE, z2.ZSERVER_FIXTURE),
    name="CastleTesting:Robot"
)


SELENIUM_FIXTURE = SeleniumCastleLayer()
SELENIUM_PLONE_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(SELENIUM_FIXTURE, CASTLE_FIXTURE, MOCK_MAILHOST_FIXTURE),
    name="SeleniumTesting:Functional")


class ResponseWrapper:
    '''Decorates a response object with additional introspective methods.'''

    _bodyre = re.compile('\r\n\r\n(.*)', re.MULTILINE | re.DOTALL)

    def __init__(self, response, outstream, path):
        self._response = response
        self._outstream = outstream
        self._path = path

    def __getattr__(self, name):
        return getattr(self._response, name)

    @property
    def output(self):
        '''Returns the complete output, headers and all.'''
        return self._outstream.getvalue()

    @property
    def body(self):
        '''Returns the page body, i.e. the output par headers.'''
        body = self._bodyre.search(self.getOutput())
        if body is not None:
            body = body.group(1)
        return body

    @property
    def path(self):
        '''Returns the path used by the request.'''
        return self._path

    @property
    def status_code(self):
        return self._response.getStatus()

    def getHeader(self, name):
        '''Returns the value of a response header.'''
        return self.headers.get(name.lower())

    def getCookie(self, name):
        '''Returns a response cookie.'''
        return self.cookies.get(name)


class BaseTest(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    def publish(self, path, basic=None, env=None, extra=None,
                request_method='GET', stdin=None, handle_errors=True):
        """
        Mostly pulled from Testing.functional
        """
        from ZPublisher.Request import Request
        from ZPublisher.Response import Response
        from ZPublisher.Publish import publish_module

        transaction.commit()

        if env is None:
            env = {}
        if extra is None:
            extra = {}

        env['SERVER_NAME'] = self.request['SERVER_NAME']
        env['SERVER_PORT'] = self.request['SERVER_PORT']
        env['REQUEST_METHOD'] = request_method

        p = path.split('?')
        if len(p) == 1:
            env['PATH_INFO'] = p[0]
        elif len(p) == 2:
            [env['PATH_INFO'], env['QUERY_STRING']] = p
        else:
            raise TypeError('')

        if basic:
            env['HTTP_AUTHORIZATION'] = "Basic %s" % base64.encodestring(basic)

        if stdin is None:
            stdin = StringIO()

        outstream = StringIO()
        response = Response(stdout=outstream, stderr=sys.stderr)
        request = Request(stdin, env, response)
        if extra:
            # Needed on Plone 3 when adding things to the path in a querystring
            # is not enough.
            for key, value in extra.items():
                request[key] = value

        publish_module('Zope2',
                       debug=not handle_errors,
                       request=request,
                       response=response)

        return ResponseWrapper(response, outstream, path)



import json


_task_descriptors = {
    'collective.celery.paste_items': 'Paste large number of items',
    'collective.celery.delete_items': 'Delete large number of items',
    'collective.celery.file_edited': 'Processing file',
    'collective.celery._celeryQueueJob': 'Converting file to document viewer',
    'collective.celery.aws_file_deleted': 'Remove file from AWS',
    'collective.celery.create_pdf': 'Create PDF',
    'collective.celery.index_batch_async': 'Index content',
    'collective.celery.process_video': 'Process video',
    'collective.celery.workflow_updated': 'Processing workflow',
    'collective.celery.trash_tree': 'Processing trash',
}


def get_task_name(_id):
    if _id in _task_descriptors:
        return _task_descriptors[_id]
    return _id


def get_info(task):
    try:
        kwargs = json.loads(task.get('kwargs', '{}').replace("'", '"'))
    except:
        kwargs = {}

    try:
        args = json.loads(task.get('args', '{}').replace("'", '"'))
    except:
        args = []

    label = _task_descriptors.get(task['name'], 'Generic task')
    return {
        'kwargs': kwargs,
        'args': args,
        'name': task.get('name'),
        'acknowledged': task.get('acknowledged'),
        'id': task.get('id'),
        'label': label
    }



from plone.app.layout.navigation.defaultpage import getDefaultPage
from plone.app.layout.sitemap import sitemap
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getUtility


MAX_ITEMS = 50000


class SiteMapView(sitemap.SiteMapView):
    template = ViewPageTemplateFile('templates/sitemap.xml')

    def objects(self):
        """Returns the data to create the sitemap."""
        catalog = getToolByName(self.context, 'portal_catalog')
        query = {
            'sort_on': 'modified',
            'sort_limit': MAX_ITEMS,
            'sort_order': 'reverse'
        }
        utils = getToolByName(self.context, 'plone_utils')
        types = utils.getUserFriendlyTypes()
        if 'Image' in types:
            types.remove('Image')
        query['portal_type'] = types
        registry = getUtility(IRegistry)
        typesUseViewActionInListings = frozenset(
            registry.get('plone.types_use_view_action_in_listings', []))

        is_plone_site_root = IPloneSiteRoot.providedBy(self.context)
        if not is_plone_site_root:
            query['path'] = '/'.join(self.context.getPhysicalPath())

        root_page_uid = ''
        # The plone site root is not catalogued.
        if is_plone_site_root:
            loc = self.context.absolute_url()
            date = self.context.modified()
            # Comparison must be on GMT value
            modified = date.ISO8601()
            # special case, use front-page object
            try:
                page = self.context[getDefaultPage(self.context)]
                root_page_uid = IUUID(page)
                modified = page.modified().ISO8601()
                yield {
                    'loc': loc,
                    'lastmod': modified
                }
            except AttributeError:
                yield {
                    'loc': loc,
                    'lastmod': modified,
                }

        for item in catalog.searchResults(query)[:MAX_ITEMS]:  # max of 50,000 items
            if root_page_uid == item.UID or item.id == 'Members':
                continue
            loc = item.getURL()
            date = item.modified.ISO8601()
            if item.portal_type in typesUseViewActionInListings:
                loc += '/view'
            yield {
                'loc': loc,
                'lastmod': date
            }



import os
from logging import getLogger
from shutil import copyfile, rmtree
from tempfile import mkdtemp

from castle.cms.commands import avconv, md5
from castle.cms.files import aws
from plone.app.blob.utils import openBlob
from plone.namedfile import NamedBlobImage
from plone.namedfile.file import NamedBlobFile

logger = getLogger('wildcard.media')


def switchFileExt(filename, ext):
    filebase = filename.rsplit('.', 1)[0]
    return filebase + '.' + ext


def process(context):
    video = context.file

    if not video or video.filename == aws.FILENAME:
        return

    try:
        opened = openBlob(video._blob)
        bfilepath = opened.name
        opened.close()
    except IOError:
        logger.warn('error opening blob file')
        return

    # by default, assume all non-mp4 videos need to be converted
    # but in reality, all videos need converting, even mp4.
    # md5 is only what makes this possible
    convert_it = video.contentType.split('/')[-1] != 'mp4'
    if md5 is not None:
        old_hash = getattr(context, '_file_hash', None)
        current_hash = md5(bfilepath)
        if old_hash is None or old_hash != current_hash:
            convert_it = True

    if context.image and not convert_it:
        # already an mp4 and already has a screen grab
        return

    tmpdir = mkdtemp()
    tmpfilepath = os.path.join(tmpdir, video.filename)
    copyfile(bfilepath, tmpfilepath)

    if convert_it:
        output_filepath = os.path.join(tmpdir, 'output.mp4')
        try:
            avconv.convert(tmpfilepath, output_filepath)
        except:
            pass
        if os.path.exists(output_filepath) and os.path.getsize(output_filepath) > 0:
            if md5 is not None:
                try:
                    context._file_hash = md5(output_filepath)
                except:
                    pass
            context._p_jar.sync()
            fi = open(output_filepath)
            namedblob = NamedBlobFile(
                fi, filename=switchFileExt(video.filename, 'mp4'))
            context.file = namedblob
            fi.close()

    if not context.image:
        # try and grab one from video
        output_filepath = os.path.join(tmpdir, u'screengrab.png')
        try:
            avconv.grab_frame(tmpfilepath, output_filepath)
            if os.path.exists(output_filepath):
                fi = open(output_filepath)
                context.image = NamedBlobImage(fi, filename=u'screengrab.png')
                fi.close()
        except:
            logger.warn('error getting thumbnail from video')
    rmtree(tmpdir)



from zope.annotation.interfaces import IAnnotations


COUNT_ANNOTATION_KEY = 'castle.socialcounts'


def get_stats(obj):
    annotations = IAnnotations(obj)
    return annotations.get(COUNT_ANNOTATION_KEY)


from castle.cms.tiles.base import BaseTile
from plone.autoform import directives as form
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from zope import schema
from zope.interface import Interface


class SearchTile(BaseTile):

    @property
    def portal_types(self):
        return self.data.get('portal_types') or []


class ISearchTileSchema(Interface):
    form.widget('portal_types', CheckBoxFieldWidget)
    portal_types = schema.List(
        title=u'Types to search',
        description=u'Leave empty to allow searching all types',
        required=False,
        default=[],
        missing_value=[],
        value_type=schema.Choice(
            vocabulary="castle.cms.vocabularies.ReallyUserFriendlyTypes"))

    placeholder = schema.TextLine(
        title=u'Search input placeholder field',
        default=u'Search...',
        required=True
    )

    search_btn_text = schema.TextLine(
        title=u'Search button text',
        default=u'Search',
        required=True
    )



# -*- coding: utf-8 -*-
#
# We patch plone.app.theming because we are overriding
# how normal plone theming works
#
#
from Acquisition import aq_parent
from castle.cms.utils import get_context_from_request
from chameleon import PageTemplate
from chameleon import PageTemplateLoader
from lxml import etree
from lxml.html import fromstring
from lxml.html import tostring
from plone import api
from plone.app.blocks import tiles
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.app.blocks.utils import replace_with_children
from plone.app.layout.globals.interfaces import IViewView
from plone.app.theming.interfaces import THEME_RESOURCE_NAME
from plone.app.theming.policy import ThemingPolicy
from plone.app.theming.utils import theming_policy
from plone.dexterity.interfaces import IDexterityContent
from plone.resource.interfaces import IResourceDirectory
from plone.resource.utils import queryResourceDirectory
from Products.CMFCore.interfaces import ISiteRoot
from repoze.xmliter.serializer import XMLSerializer
from repoze.xmliter.utils import getHTMLSerializer
from urlparse import urljoin
from zExceptions import NotFound
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides

import Globals
import logging
import json
import re

logger = logging.getLogger('castle.cms')

OVERRIDE_ENVIRON_KEY = 'castle.override.theme'
wrapper_xpath = etree.XPath('//*[@id="visual-portal-wrapper"]')
href_xpath = etree.XPath('//*[@href]')
src_xpath = etree.XPath('//*[@src]')
js_insert_xpath = etree.XPath('//link[@data-include-js]')
css_insert_xpath = etree.XPath('//link[@data-include-css]')
script_xpath = etree.XPath('//script')
body_xpath = etree.XPath('/html/body')
head_xpath = etree.XPath('/html/head')
contentpanel_xpath = etree.XPath("//*[@data-panel='content']")
content_xpath = etree.XPath('//*[@id="content"]')
column1_xpath = etree.XPath('//*[@id="portal-column-one"]')
column2_xpath = etree.XPath('//*[@id="portal-column-two"]')
jsslot_xpath = etree.XPath('//*[@id="javascript_head_slot"]')
styleslot_xpath = etree.XPath('//*[@id="style_slot"]')
dynamic_grid_xpath = etree.XPath('//*[@dynamic-grid]')
LAYOUT_NAME = re.compile(r'[a-zA-Z_\-]+/[a-zA-Z_\-]+')


class ThemeTemplateLoader(PageTemplateLoader):

    def __init__(self, theme, template_cache=None, *args, **kwargs):
        self.file_cache = {}
        if template_cache is None:
            template_cache = {}
        self.template_cache = template_cache
        self.theme = theme
        try:
            self.folder = queryResourceDirectory(THEME_RESOURCE_NAME, theme)
        except:
            self.folder = None
        super(PageTemplateLoader, self).__init__(*args, **kwargs)

    def load(self, filename, backup='index.html'):
        """Load and return a template file.

        The format parameter determines will parse the file. Valid
        options are `xml` and `text`.
        """
        if filename in self.template_cache:
            return self.template_cache[filename]

        try:
            data = self.read_file(filename)
        except:
            data = None
        if not data:
            filename = backup
            if filename in self.template_cache:
                return self.template_cache[filename]
            data = self.read_file(filename)

        template = PageTemplate(data)
        self.template_cache[filename] = template
        return template

    __getitem__ = load

    def read_file(self, filename):
        if self.folder is None:
            return
        if filename in self.file_cache:
            return self.file_cache[filename]
        try:
            if isinstance(filename, unicode):
                filename = filename.encode('utf8')
            result = unicode(self.folder.readFile(filename), 'utf8')
            self.file_cache[filename] = result
            return result
        except (NotFound, IOError):
            raise KeyError

    def load_raw(self, raw):
        return PageTemplate(raw)


def join(base, url):
    """
    Join relative URL
    """
    if not (url.startswith("/") or "://" in url):
        return urljoin(base, url)
    else:
        # Already absolute
        return url


def replace_el(el, replacement):
    el.getparent().replace(el, replacement)


def add_children(el, with_children):
    for child in with_children.getchildren():
        el.append(child)


class _Transform(object):
    """
    Warning: This object is being cached in a thread local so we can NOT store persistent
    data anywhere here...
    """
    def __init__(self, name):
        self.name = name or 'castle.theme'  # provide backup theme in case missing
        self.template_cache = {}

    def __call__(self, request, result, context=None):
        if '++plone++' in request.ACTUAL_URL:
            return
        portal = api.portal.get()
        original_context = context
        if context is None or IResourceDirectory.providedBy(context):
            original_context = context = portal

        try:
            context_url = context.absolute_url()
        except AttributeError:
            # could be a form/browser class
            try:
                context = context.context
            except:
                context = aq_parent(context)
            context_url = context.absolute_url()

        if (not ISiteRoot.providedBy(context) and
                not IDexterityContent.providedBy(context)):
            context = aq_parent(context)
        portal_url = portal.absolute_url()

        raw = False
        if isinstance(result, basestring):
            raw = True
        else:
            result = result.tree

        theme_base_url = '%s/++%s++%s/index.html' % (
            portal_url,
            THEME_RESOURCE_NAME,
            self.name)

        content = self.get_fill_content(result, raw)

        utils = getMultiAdapter((context, request),
                                name='castle-utils')

        layout = self.get_layout(context, request=request)
        layout = layout(
            portal_url=portal_url,
            site_url=portal_url,
            context_url=context_url,
            request=request,
            context=context,
            portal=portal,
            site=portal,
            theme_base_url=theme_base_url,
            content=content,
            anonymous=api.user.is_anonymous(),
            debug=api.env.debug_mode(),
            utils=utils
        )

        dom = getHTMLSerializer([layout])
        self.rewrite(dom, theme_base_url)
        if not raw:
            # old style things...
            self.bbb(dom.tree, result)

        dom.tree = tiles.renderTiles(request, dom.tree)

        self.add_body_classes(original_context, context, request,
                              dom.tree, result, raw)

        self.add_included_resources(dom.tree, portal, request)
        self.dynamic_grid(dom.tree)
        # #
        # dom.tree = tiles.renderTiles(request, dom.tree)

        return dom

    def add_viewlet_tile(self, portal, request, el, name):
        tile = queryMultiAdapter(
            (portal, request), name=name)
        alsoProvides(tile, IViewView)
        if tile:
            viewlet = tile.get_viewlet(tile.manager, tile.viewlet)
            if viewlet:
                viewlet.update()
                replace_with_children(el, fromstring(viewlet.render()))

    def add_included_resources(self, dom, portal, request):
        # if theme has js/css drop points instead of using those tiles
        el = js_insert_xpath(dom)
        if len(el) > 0:
            self.add_viewlet_tile(portal, request, el[0],
                                  'plone.app.standardtiles.javascripts')
        el = css_insert_xpath(dom)
        if len(el) > 0:
            self.add_viewlet_tile(portal, request, el[0],
                                  'plone.app.standardtiles.stylesheets')

    def get_loader(self):
        return ThemeTemplateLoader(self.name, template_cache=self.template_cache)

    def get_raw_layout(self, context, loader=None):
        """not compiled"""
        if loader is None:
            loader = self.get_loader()
        layout_name = self.get_layout_name(context)
        try:
            return loader.read_file(layout_name)
        except KeyError:
            if layout_name != 'index.html':
                # try backup
                try:
                    return loader.read_file('index.html')
                except KeyError:
                    pass

    def get_layout_name(self, context, default_layout='index.html'):
        adapted = ILayoutAware(context, None)
        selected = None
        if adapted is not None:
            selected = adapted.pageSiteLayout
            if selected is None:
                context = aq_parent(context)
                while not ISiteRoot.providedBy(context):
                    adapted = ILayoutAware(context, None)
                    if adapted and adapted.sectionSiteLayout:
                        selected = adapted.sectionSiteLayout
                        break
                    context = aq_parent(context)

        if selected is None:
            if ISiteRoot.providedBy(context):
                # check if a default layout is set
                try:
                    selected = context.portal_registry['castle.cms.default_layout'] or default_layout  # noqa
                except (AttributeError, KeyError):
                    selected = default_layout
            else:
                selected = default_layout
        return selected

    def get_layout(self, context, default_layout='index.html', request=None, loader=None):
        if loader is None:
            loader = self.get_loader()
        if request is not None and 'X-CASTLE-LAYOUT' in request.environ:
            layout = loader.load_raw(request.environ['X-CASTLE-LAYOUT'])
            selected_name = 'environ'
        else:
            selected_name = self.get_layout_name(context, default_layout)

            try:
                layout = loader[selected_name]
	    except Exception as e:
		logger.error('Failed parsing content layout', exc_info=True)
		layout = None

            if layout is None:
                # default to 'index.html' now
                layout = loader['index.html']

        layout.name = selected_name
        return layout

    def add_body_classes(self, original_context, context, request, tree, result, raw=False):
        body_classes = ''
        if raw:
            # this is a content layout likely
            plone_layout = queryMultiAdapter(
                (original_context, request), name='plone_layout')
            if plone_layout:
                body_classes += ' ' + plone_layout.bodyClass(None, None)
        else:
            content_body = body_xpath(result)
            if len(content_body) > 0:
                body_classes += ' ' + content_body[0].attrib.get('class', '')

        body = body_xpath(tree)[0]
        from plone.app.blocks.layoutbehavior import ILayoutAware
        if ILayoutAware.providedBy(context):
            body_classes += ' template-layout'
            adapted = ILayoutAware(context, None)
            if adapted is not None:
                layout = getattr(adapted, 'contentLayout', None)
                if layout:
                    # Transform ++contentlayout++default/document.html
                    # into layout-default-document
                    names = LAYOUT_NAME.findall(layout)
                    if len(names) == 1:
                        body_classes += ' layout-' + names[0].replace('/', '-')
                else:
                    body_classes += ' layout-custom'

        try:
            body_classes += ' selected-layout-%s ' % original_context.getLayout()
        except:
            pass
        classes = '%s %s' % (
            body.attrib.get('class', ''),
            body_classes
        )
        body.attrib['class'] = classes

        plone_view = queryMultiAdapter(
            (original_context, request), name='plone')
        if plone_view:
            try:
                body.attrib.update(plone_view.patterns_settings())
            except AttributeError:
                plone_view = queryMultiAdapter(
                    (context, request), name='plone')
                if plone_view:
                    body.attrib.update(plone_view.patterns_settings())

    def rewrite(self, dom, base_url):
        for node in href_xpath(dom.tree):
            url = node.get('href')
            if url:
                url = join(base_url, url)
                node.set('href', url)
        for node in src_xpath(dom.tree):
            url = node.get('src')
            if url:
                url = join(base_url, url)
                node.set('src', url)

    def bbb(self, dom, result):
        """
        old style page template, do some bbb things here.
        1) implement javascript_head_slot
        2) implement style_slot
        """
        # add children of javascript_head_slot after all other
        # javascript on the page
        head = head_xpath(dom)
        if len(head) > 0:
            js_slot = jsslot_xpath(result)
            if len(js_slot) > 0:
                scripts = script_xpath(dom)
                if len(scripts) > 0:
                    script = scripts[-1]  # insert after last script on page
                    parent = script.getparent()
                    script_idx = parent.index(script)
                    for idx, child in enumerate(js_slot[0].getchildren()):
                        parent.insert(script_idx + 1 + idx, child)
                else:
                    add_children(head[0], js_slot[0])
            style_slot = styleslot_xpath(result)
            if len(style_slot) > 0:
                add_children(head[0], style_slot[0])

    def get_fill_content(self, result, raw=False):
        """
        get main, left and right columns so we can potentially
        render them in the template
        """
        main_html = ''
        column1_html = ''
        column2_html = ''
        if raw:
            main_html = result
        else:
            content = contentpanel_xpath(result)
            if len(content) == 0:
                content = content_xpath(result)
            if len(content) > 0:
                main_html = tostring(content[0])

            column1 = column1_xpath(result)
            if len(column1) > 0 and len(column1[0]) > 0:
                column1_html = tostring(column1[0])
            column2 = column2_xpath(result)
            if len(column2) > 0 and len(column2[0]) > 0:
                column2_html = tostring(column2[0])
        return {
            'main': main_html,
            'left': column1_html,
            'right': column2_html
        }

    def dynamic_grid(self, dom):
        for container in dynamic_grid_xpath(dom):
            found = []
            classes = []
            for child in container.getchildren():
                # remove empties
                if len(child) == 0:
                    container.remove(child)
                else:
                    found.append(child)

            for child in found:
                if 'id' in child.attrib:
                    classes.append(child.attrib['id'])
                try:
                    width = int(child.attrib.get('col-count-%i' % len(found), '4'))
                except:
                    width = 4
                child.attrib['data-grid'] = json.dumps({
                    "type": "cell",
                    "info": {"pos": {"width": width, "x": 0}}
                })
                

            classes.append('-'.join(classes))
            classes.append('col-count-%i' % len(found))
            container.attrib['class'] = '{} {}'.format(
                container.attrib.get('class', ''),
                ' '.join(classes)
            )


def getTransform(context, request):
    DevelopmentMode = Globals.DevelopmentMode
    policy = theming_policy(request)

    # Obtain settings. Do nothing if not found
    settings = policy.getSettings()

    if settings is None:
        return None

    try:
        if not policy.isThemeEnabled():
            return None
    except:
        pass

    cache = policy.getCache()

    # Apply theme
    transform = None

    if not DevelopmentMode and cache.transform:
        transform = cache.transform

    if transform is None:
        name = policy.getCurrentTheme()
        transform = _Transform(name)

        if not DevelopmentMode:
            cache.updateTransform(transform)

    return transform


def renderWithTheme(context, request, content):
    """
    Also, this disables further theme transform since presumably
    this is already done.
    """
    transform = getTransform(context, request)
    if transform:
        request.response.setHeader('X-Theme-Applied', 'true')
        return transform(request, content, context=context)
    else:
        return content


class Policy(ThemingPolicy):
    def getCurrentTheme(self):
        """The name of the current theme."""
        if OVERRIDE_ENVIRON_KEY in self.request.environ:
            return self.request.environ[OVERRIDE_ENVIRON_KEY]
        settings = self.getSettings()
        if settings.currentTheme:
            return settings.currentTheme

        return None

    def isThemeEnabled(self, settings=None):
        """Whether theming is enabled."""

        # Resolve DevelopmentMode late (i.e. not on import time) since it may
        # be set during import or test setup time
        DevelopmentMode = Globals.DevelopmentMode

        # Disable theming if the response sets a header
        if self.request.response.getHeader('X-Theme-Disabled'):
            return False

        # Check for diazo.off request parameter
        true_vals = ('1', 'y', 'yes', 't', 'true')
        if (DevelopmentMode and self.request.get(
                'diazo.off', '').lower() in true_vals):
            return False

        if not settings:
            settings = self.getSettings()
        if not settings.enabled:
            return False

        server_url = self.request.get('SERVER_URL')
        proto, host = server_url.split('://', 1)
        host = host.lower()
        serverPort = self.request.get('SERVER_PORT')

        for hostname in settings.hostnameBlacklist or ():
            if host == hostname or host == ':'.join((hostname, serverPort)):
                return False

        return True


def isPloneTheme(settings):
    return (settings.rules and
            settings.rules not in (None, '_', '') and
            settings.rules.endswith('.xml'))


def transformIterable(self, result, encoding):
    """Apply the transform if required
    """
    if self.request.response.getHeader('X-Theme-Applied'):
        return
    if not isinstance(result, XMLSerializer):
        return

    # Obtain settings. Do nothing if not found
    policy = theming_policy(self.request)
    settings = policy.getSettings()
    if settings is None:
        return None
    if not policy.isThemeEnabled():
        return None

    try:
        if isPloneTheme(settings):
            # XXX old style theme
            # manual render tiles, then do theme transform
            result.tree = tiles.renderTiles(self.request, result.tree)
            result = self._old_transformIterable(result, encoding)
            return result
    except AttributeError:
        pass

    DevelopmentMode = Globals.DevelopmentMode

    try:
        # if we are here, it means we are rendering the the
        # classic way and we need to do replacements.
        # check for #visual-portal-wrapper to make sure we need
        # transform this response
        # XXX WARNING: THIS IS NECESSARY
        wrapper = wrapper_xpath(result.tree)
        if len(wrapper) == 0:
            return None

        context = get_context_from_request(self.request)
        transform = getTransform(context, self.request)
        if transform is None:
            return None

        transformed = transform(self.request, result, context=context)
        if transformed is None:
            return None

        result = transformed
        if settings.doctype:
            result.doctype = settings.doctype
            if not result.doctype.endswith('\n'):
                result.doctype += '\n'
        return result
    except etree.LxmlError:
        if not(DevelopmentMode):
            raise
    return result



from castle.cms.commands import docsplit
from castle.cms.utils import get_data_from_url
from castle.cms.utils import inline_images_in_dom
from lxml.etree import tostring
from lxml.html import fromstring
from lxml.html.clean import Cleaner
from plone import api
from plone.app.blob.utils import openBlob
from plone.registry.interfaces import IRegistry
from ZODB.blob import Blob
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.globalrequest import getRequest

import json
import logging
import requests
import traceback


logger = logging.getLogger('castle.cms')
cleaner = Cleaner(style=True, page_structure=False,
                  processing_instructions=False, forms=False, meta=True,
                  annoying_tags=False, remove_unknown_tags=False)


def create_raw_from_view(context, view_name='pdf', css_files=[]):
    request = getRequest()
    view = getMultiAdapter((context, request), name=view_name)
    html = view()
    portal = api.portal.get()
    site_url = portal.absolute_url()
    css = []
    for css_file in css_files:
        ct, data = get_data_from_url(css_file)
        if isinstance(data, basestring):
            css.append(data)
    xml = fromstring(html)
    cleaner(xml)
    inline_images_in_dom(xml, portal, site_url)

    registry = getUtility(IRegistry)
    public_url = registry.get(
        'plone.public_url', '')

    for anchor in xml.cssselect('a'):
        if anchor.attrib.get('href', '').startswith(site_url):
            anchor.attrib['href'] = anchor.attrib['href'].replace(site_url, public_url)

    return tostring(xml), css


def create(html, css):
    try:
        registry = getUtility(IRegistry)
        prince_server_url = registry.get(
            'castle.princexml_server_url', 'http://localhost:6543/convert')
        if prince_server_url is None:
            logger.warn('error converting pdf')
            return
        logger.info('start converting pdf')
        xml = fromstring(html)
        # save styles
        resp = requests.post(
            prince_server_url,
            data={'xml': tostring(xml), 'css': json.dumps(css)})
        data = resp.content
        blob = Blob()
        bfile = blob.open('w')
        bfile.write(data)
        bfile.close()
        return blob
    except:
        logger.info(traceback.format_exc())
        raise


def screenshot(blob):
    blobfi = openBlob(blob)
    filepath = docsplit.dump_image(blobfi.read(), '1000', 'gif')
    blobfi.close()

    blob = Blob()
    bfile = blob.open('w')
    sfi = open(filepath, 'rb')
    bfile.write(sfi.read())
    bfile.close()
    sfi.close()
    return blob



from Acquisition import aq_inner
from Acquisition import aq_parent
from borg.localrole.interfaces import IFactoryTempFolder
from castle.cms import cache
from castle.cms import theming
from castle.cms.interfaces import IDashboard
from plone import api
from plone.app.imaging.utils import getAllowedSizes
from plone.app.layout.navigation.defaultpage import getDefaultPage
from plone.dexterity.interfaces import IDexterityContainer
from plone.memoize.view import memoize
from plone.registry.interfaces import IRegistry
from plone.tiles.interfaces import ITileType
from plone.uuid.interfaces import IUUID
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.interfaces._content import IFolderish
from Products.CMFPlone.interfaces import IPatternsSettings
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.CMFPlone.patterns import PloneSettingsAdapter
from Products.CMFPlone.patterns import TinyMCESettingsGenerator
from zope.component import getUtility
from zope.interface import implements

import json


class CastleTinyMCESettingsGenerator(TinyMCESettingsGenerator):

    def get_tiny_config(self):
        config = super(CastleTinyMCESettingsGenerator, self).get_tiny_config()
        if 'paste' not in config['plugins']:
            config['plugins'] += 'paste'
        config['paste_data_images'] = True
        config['paste_as_text'] = True
        config['paste_word_valid_elements'] = "b,strong,i,em,h1,h2,h3,h4,ul,li"
        config['paste_retain_style_properties'] = ''
        config['paste_merge_formats'] = True
        content_css = config['content_css'].split(',')
        style_css = config['importcss_file_filter'].split(',')
        content_css.extend(style_css)
        config['content_css'] = ','.join(set(content_css))
        config['browser_spellcheck'] = True
        formats = config.get('formats')
        if isinstance(formats, list):
            config['style_formats'].extend(formats)
        return config


class CastleSettingsAdapter(PloneSettingsAdapter):
    """
    This adapter will handle all default plone settings.

    Right now, it only does tinymce
    """
    implements(IPatternsSettings)

    def __init__(self, context, request, field):
        super(CastleSettingsAdapter, self).__init__(context, request, field)
        self.site = api.portal.get()

    @property
    @memoize
    def registry(self):
        return getUtility(IRegistry)

    def get_available_slot_tiles(self):
        cache_key = '%s-slot-tiles' % '/'.join(self.site.getPhysicalPath()[1:])
        try:
            return cache.get(cache_key)
        except:
            pass

        available_tiles = self.registry.get('castle.slot_tiles')
        if not available_tiles:
            available_tiles = {
                'Structure': ['plone.app.standardtiles.rawhtml']
            }

        # otherwise, you're editing the value in the DB!!!!
        available_tiles = available_tiles.copy()

        for group_name, tile_ids in available_tiles.items():
            group = []
            for tile_id in tile_ids:
                tile = getUtility(ITileType, name=tile_id)
                group.append({
                    'id': tile_id,
                    'label': tile.title
                })
            available_tiles[group_name] = group
        cache.set(cache_key, available_tiles, 600)
        return available_tiles

    def tinymce(self):
        if api.user.is_anonymous():
            return {}
        generator = CastleTinyMCESettingsGenerator(self.context, self.request)
        settings = generator.settings

        folder = aq_inner(self.context)
        # Test if we are currently creating an Archetype object
        if IFactoryTempFolder.providedBy(aq_parent(folder)):
            folder = aq_parent(aq_parent(aq_parent(folder)))
        if not IFolderish.providedBy(folder):
            folder = aq_parent(folder)

        if IPloneSiteRoot.providedBy(folder):
            initial = None
        else:
            initial = IUUID(folder, None)
        current_path = folder.absolute_url()[len(generator.portal_url):]

        scales = []
        for name, info in sorted(getAllowedSizes().items(), key=lambda x: x[1][0]):
            scales.append({
                'part': name,
                'name': name,
                'label': '{} ({}x{})'.format(
                    name.capitalize(),
                    info[0],
                    info[1])
            })
        image_types = settings.image_objects or []
        folder_types = settings.contains_objects or []
        configuration = {
            'relatedItems': {
                'vocabularyUrl':
                    '%s/@@getVocabulary?name=plone.app.vocabularies.Catalog' % (
                        generator.portal_url)
            },
            'upload': {
                'initialFolder': initial,
                'currentPath': current_path,
                'baseUrl': generator.portal_url,
                'relativePath': '@@fileUpload',
                'uploadMultiple': False,
                'maxFiles': 1,
                'showTitle': False
            },
            'base_url': self.context.absolute_url(),
            'tiny': generator.get_tiny_config(),
            # This is for loading the languages on tinymce
            'loadingBaseUrl': '%s/++plone++static/components/tinymce-builded/js/tinymce' % generator.portal_url,  # noqa
            'prependToUrl': 'resolveuid/',
            'linkAttribute': 'UID',
            'prependToScalePart': '/@@images/image/',
            'folderTypes': folder_types,
            'imageTypes': image_types,
            'scales': scales
        }

        return {'data-pat-tinymce': json.dumps(configuration)}

    def __call__(self):
        data = super(CastleSettingsAdapter, self).__call__()

        if api.user.is_anonymous():
            return data

        folder = self.context
        if not IDexterityContainer.providedBy(folder):
            folder = aq_parent(folder)
        required_upload_fields = self.registry.get(
            'castle.required_file_upload_fields', []) or []
        data.update({
            'data-available-slots': json.dumps(self.get_available_slot_tiles()),
            'data-required-file-upload-fields': json.dumps(required_upload_fields),
            'data-google-maps-api-key': self.registry.get(
                'castle.google_maps_api_key', '') or '',
            'data-folder-url': folder.absolute_url()
        })

        show_tour = False
        user = api.user.get_current()
        viewed = user.getProperty('tours_viewed', [])
        if ('all' not in viewed and
                set(viewed) != set(['welcome', 'dashboard', 'foldercontents',
                                    'addcontentinitial', 'addcontentadd', 'editpage'])):
            show_tour = True

        if show_tour and not api.env.test_mode():
            data['data-show-tour'] = json.dumps({
                'viewed': viewed
            })

        folder = self.context
        if not ISiteRoot.providedBy(folder) and not IDexterityContainer.providedBy(folder):
            folder = aq_parent(folder)
        site_path = self.site.getPhysicalPath()
        folder_path = folder.getPhysicalPath()
        data['data-base-path'] = '/' + '/'.join(folder_path[len(site_path):])

        real_context = self.context
        if ISiteRoot.providedBy(real_context):
            # we're at site root but actually kind of want context front page
            try:
                real_context = real_context[getDefaultPage(real_context)]
            except (AttributeError, KeyError):
                pass
        if IDashboard.providedBy(real_context):
            real_context = self.site

        transform = theming.getTransform(real_context, self.request)
        if transform is not None:
            data['data-site-layout'] = transform.get_layout_name(real_context)

        data['data-site-default'] = getDefaultPage(self.site) or 'front-page'

        return data



from castle.cms import social
from castle.cms.tiles.base import BaseTile
from plone.autoform import directives as form
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from zope import schema
from zope.interface import Interface
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class SharingTile(BaseTile):

    def render(self):
        return self.index()

    @property
    def buttons(self):
        return self.data.get('buttons',
                             ['twitter', 'facebook', 'email', 'print', 'share'])

    def get_counts(self):
        return social.get_stats(self.context)


class ISharingTileSchema(Interface):

    form.widget('buttons', CheckBoxFieldWidget)
    buttons = schema.List(
        title=u'Buttons',
        description=u'Types of share buttons',
        value_type=schema.Choice(
            vocabulary=SimpleVocabulary([
                SimpleTerm('twitter', 'twitter', 'Twitter'),
                SimpleTerm('facebook', 'facebook', 'Facebook'),
                SimpleTerm('email', 'email', 'Email'),
                SimpleTerm('print', 'print', 'Print'),
                SimpleTerm('share', 'share', 'Share')
            ])
        ),
        default=['twitter', 'facebook', 'email', 'print', 'share']
    )



from castle.cms.tiles.base import BaseTile
from castle.cms.utils import parse_query_from_data
from castle.cms.widgets import QueryFieldWidget
from datetime import timedelta
from plone.autoform import directives as form
from plone.event.utils import is_datetime
from plone.event.utils import pydt
from Products.CMFCore.utils import getToolByName
from zope import schema
from zope.interface import Interface

import json


def format_date(dt):
    if is_datetime(dt):
        return dt.isoformat()
    else:
        try:
            return dt.ISO8601()
        except:
            pass


class CalendarTile(BaseTile):

    def get_query(self):
        parsed = parse_query_from_data(self.data, self.context)
        parsed['sort_order'] = 'reverse'
        return parsed

    def results(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        return catalog(**self.get_query())[:self.data.get('limit', 250) or 250]

    def json_config(self):
        events = []
        for brain in self.results():
            event = {
                'title': brain.Title,
                'url': brain.getURL()
            }
            if brain.portal_type == 'Event':
                event.update({
                    'start': brain.start and format_date(brain.start),
                    'end': brain.end and format_date(brain.end)
                })
            else:
                event['start'] = format_date(brain.effective)
                if brain.expires and brain.expires.year() != 2499:
                    event['end'] = format_date(brain.expires)
                else:
                    event['end'] = format_date(pydt(brain.effective) + timedelta(hours=1))
            events.append(event)

        return json.dumps({
            'header': {
                'left': 'prev,next today',
                'center': 'title',
                'right': 'month,agendaWeek,agendaDay'
            },
            'events': events
        })


class ICalendarTileSchema(Interface):

    form.widget(query=QueryFieldWidget)
    query = schema.List(
        title=u'Base query',
        description=u"This query can be customized base on user selection",
        value_type=schema.Dict(value_type=schema.Field(),
                               key_type=schema.TextLine()),
        required=False,
        default=[{
            u'i': u'portal_type',
            u'v': [u'Event'],
            u'o': u'plone.app.querystring.operation.selection.any'}]
    )

    sort_on = schema.TextLine(
        title=u'Sort on',
        description=u"Sort on this index",
        required=False,
    )

    sort_reversed = schema.Bool(
        title=u'Reversed order',
        description=u'Sort the results in reversed order',
        required=False,
    )

    limit = schema.Int(
        title=u'Limit',
        description=u'Limit Search Results',
        required=False,
        default=250,
        min=1,
    )



from castle.cms.pdf.generator import create_raw_from_view
from castle.cms.pdf.generator import create
from castle.cms.pdf.generator import screenshot
from castle.cms.settings import PDFSetting
from castle.cms.utils import retriable
from collective.celery import task
import time


@retriable()
def _create_pdf(obj, html, css):
    blob = create(html, css)
    sblob = screenshot(blob)
    obj._p_jar.sync()
    settings = PDFSetting(obj)
    settings.put(blob)
    settings.put_screenshot(sblob)


@task.as_admin()
def create_pdf(obj, html, css):
    # this completes so fast we get conflict errors on save sometimes.
    # just cool it a bit
    time.sleep(2)
    obj._p_jar.sync()
    return _create_pdf(obj, html, css)


@task.as_admin()
def create_pdf_from_view(obj, css_files=[]):
    # this completes so fast we get conflict errors on save sometimes.
    # just cool it a bit
    time.sleep(2)
    obj._p_jar.sync()
    html, css = create_raw_from_view(obj, css_files=css_files)
    return _create_pdf(obj, html, css)



from zope.globalrequest import getRequest
from zope.annotation.interfaces import IAnnotations
from plone.memoize.interfaces import ICacheChooser
from plone.memoize.ram import choose_cache as base_choose_cache
from plone.memoize.ram import AbstractDict
from zope.interface import directlyProvides

import cPickle
import os
import redis
import threading

from zope import ramcache


global_ram_cache = ramcache.ram.RAMCache()
global_ram_cache.update(maxAge=60 * 10)


thread_local = threading.local()


def redis_installed():
    try:
        return isinstance(get_client(), RedisAdapter)
    except:
        return False


class RedisAdapter(AbstractDict):
    """
    Also cache ON the request to deal with some cache calls that are done
    multiple times per request
    """

    def __init__(self, client, globalkey=''):
        self.client = client
        self.globalkey = globalkey and '%s:' % globalkey or ''

    def _make_key(self, source):
        if isinstance(source, unicode):
            source = source.encode('utf-8')
        return source

    def get_key(self, key):
        return self.globalkey + self._make_key(key)

    def __getitem__(self, key):
        if 'plone.app.theming.plugins' in key:
            # bit of a hack here... We don't want theming plugins to be
            # contacting redis for every call. This is just a utility lookup
            # so it is cheaper to do it in code than cache with redis and
            # potentially have the network traffic
            raise KeyError(key)

        cached_value = self.client.get(key)
        if cached_value is None:
            raise KeyError(key)
        else:
            val = cPickle.loads(cached_value)
            return val

    def __setitem__(self, key, value):
        cache_key = self.get_key(key)
        try:
            cached_value = cPickle.dumps(value)
            self.client.set(cache_key, cached_value)
        except cPickle.PicklingError:
            pass


def get_client(fun_name=''):
    server = os.environ.get('REDIS_SERVER', None)
    if server is None:
        return base_choose_cache(fun_name)

    client = getattr(thread_local, "client", None)
    if client is None:
        server = os.environ.get("REDIS_SERVER", "127.0.0.1:6379")
        host, port = server.split(':')
        client = redis.StrictRedis(host=host, port=int(port), db=0)
        try:
            client.get('test-key')
            thread_local.client = client
        except redis.exceptions.ConnectionError:
            return base_choose_cache(fun_name)
    return RedisAdapter(client, fun_name)

directlyProvides(get_client, ICacheChooser)


def get(name):
    return get_client()[name]


def set(name, value, expire=None):
    client = get_client()
    client[name] = value
    if expire and isinstance(client, RedisAdapter):
        try:
            key = client.get_key(name)
            client = client.client
            client.expire(key, expire)
        except AttributeError:
            pass


def delete(name):
    try:
        get_client().client.delete(name)
    except AttributeError:
        pass


MARKER = object()


class RamCache(object):
    def get(self, name):
        value = global_ram_cache.query('', {'key': name}, default=MARKER)
        if value is MARKER:
            raise KeyError(name)
        return value

    def set(self, name, value):
        global_ram_cache.set(value, '', {'key': name})
ram = RamCache()



from castle.cms._scripts.utils import get_module_dir
from castle.cms.cron.utils import get_sites
from castle.cms.cron.utils import setup_site
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces import IBundleRegistry
from Products.CMFPlone.interfaces import IResourceRegistry
from zope.component import getUtility

import castle.cms
import json
import os
import Products.CMFPlone


DIR = os.path.dirname(os.path.realpath(__file__))
WORK_DIR = os.path.join(os.getcwd(), 'devtools')


if not os.path.exists(WORK_DIR):
    os.mkdir(WORK_DIR)


def jdump(data):
    return json.dumps(data, sort_keys=True,
                      indent=4, separators=(',', ': '))


def add_quotes(ss):
    return '"' + ss + '"'


SCRIPT_DIR = os.path.join(get_module_dir(castle.cms), '_scripts')
CMFPlone_DIR = get_module_dir(Products.CMFPlone)

webpack_aliases = {}
bundles_config = {}


for site in get_sites(app):  # noqa
    setup_site(site)
    os.environ['SITE_ID'] = site.getId()
    break


registry = getUtility(IRegistry)
bundles = registry.collectionOfInterface(
    IBundleRegistry, prefix="plone.bundles", check=False)
resources = registry.collectionOfInterface(
    IResourceRegistry, prefix="plone.resources", check=False)


with open(os.path.join(SCRIPT_DIR, 'templates/watchable-grunt.js')) as fi:
    tmpl = fi.read()
    with open('watchable-grunt.js', 'w') as output:
        output.write(tmpl % {
            'castle': os.path.join(get_module_dir(castle.cms), 'static')
        })


with open(os.path.join(SCRIPT_DIR, 'templates/package.json')) as fi:
    tmpl = fi.read()
    with open('package.json', 'w') as output:
        output.write(tmpl)


with open(os.path.join(SCRIPT_DIR, 'templates/Makefile')) as fi:
    tmpl = fi.read()
    with open('Makefile', 'w') as output:
        output.write(tmpl)


with open(os.path.join(SCRIPT_DIR, 'templates/watch-run.py')) as fi:
    tmpl = fi.read()
    with open('watch-run.py', 'w') as output:
        output.write(tmpl)


print('generate original grunt file')

with open(os.path.join(CMFPlone_DIR, '_scripts/_generate_gruntfile.py')) as fi:
    code = fi.read()
    exec(code, globals(), locals())
# __import__('Products.CMFPlone._scripts._generate_gruntfile', globals(), locals())



# -*- coding: utf-8 -*-
from castle.cms.tiles.base import BaseTile
from castle.cms.utils import parse_query_from_data
from castle.cms.widgets import QueryFieldWidget
from castle.cms.widgets import RelatedItemFieldWidget
from castle.cms.widgets import UseQueryWidget
from plone.autoform import directives as form
from plone.supermodel import model
from plone.tiles.interfaces import IPersistentTile
from Products.CMFCore.utils import getToolByName
from zope import schema
from zope.interface import implements
from zope.interface import Invalid
from zope.interface import invariant


class ContentTile(BaseTile):
    implements(IPersistentTile)
    default_display_fields = ('title', 'image', 'description')

    def render(self):
        return self.index()

    @property
    def content(self):
        if self.data.get('use_query') in ('True', True, 'true'):
            catalog = getToolByName(self.context, 'portal_catalog')
            items = catalog(**self.query)
            if len(items) > 0:
                return items[0].getObject()
        else:
            return self.utils.get_object(self.data['content'][0])

    @property
    def query(self):
        parsed = parse_query_from_data(self.data, self.context)
        parsed['sort_limit'] = 1
        return parsed

    @property
    def display_fields(self):
        df = self.data.get('display_fields', None)
        if df is None:
            df = self.default_display_fields
        return df


class IContentTileSchema(model.Schema):

    form.widget(use_query=UseQueryWidget)
    use_query = schema.Bool(
        title=u'Use dynamic query',
        default=False)

    form.widget(content=RelatedItemFieldWidget)
    content = schema.List(
        title=u"Item",
        description=u"Selected item to display",
        required=False,
        default=[],
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.Catalog'
        )
    )

    @invariant
    def validate_content(data):
        if data.content and len(data.content) != 1:
            raise Invalid("Must select 1 item")

    form.widget(query=QueryFieldWidget)
    query = schema.List(
        title=u'Search terms',
        description=u"Define the search terms for the item you want "
                    u"to list by choosing what to match on. "
                    u"The list of results will be dynamically updated",
        value_type=schema.Dict(value_type=schema.Field(),
                               key_type=schema.TextLine()),
        required=False
    )

    sort_on = schema.TextLine(
        title=u'Sort on',
        description=u"Sort on this index",
        required=False,
    )

    sort_reversed = schema.Bool(
        title=u'Reversed order',
        description=u'Sort the results in reversed order',
        required=False,
    )

    more_text = schema.TextLine(
        title=u'More text',
        default=u'',
        required=False)



# -*- coding: utf-8 -*-
from DateTime import DateTime
from plone.app.theming.interfaces import THEME_RESOURCE_NAME
from plone.app.theming.utils import getCurrentTheme
from plone.resource.utils import queryResourceDirectory
from z3c.jbot import interfaces
from z3c.jbot.manager import find_package
from zope import interface
from zope.component import ComponentLookupError
from zope.component.hooks import getSite
from zope.filerepresentation.interfaces import IRawReadFile
from zope.globalrequest import getRequest

import Globals
import logging
import os
import sys
import traceback


REQ_CACHE_KEY = 'castle.cms.jbot.storage'

logger = logging.getLogger('castle.cms')
IGNORE = object()


_file_cache = {}


class Storage(object):

    def __init__(self):
        self.theme = getCurrentTheme()
        self.site = getSite()
        if self.theme:
            directory = queryResourceDirectory(
                THEME_RESOURCE_NAME, self.theme)
            try:
                self.themeDirectory = directory['jbot']
            except:
                self.themeDirectory = None
        else:
            self.themeDirectory = None

    def __contains__(self, name):
        if self.themeDirectory:
            if name in self.themeDirectory:
                return True
        return False

    def _get_fs_path(self):
        """ returns FS path for jbot storage.
        """
        base_path = '/'.join(Globals.data_dir.split('/')[:-1])
        jbot_dir = os.path.join(base_path, 'jbot')
        site_id = self.site.getId()
        if not os.path.exists(jbot_dir):
            os.makedirs(jbot_dir)
            logger.info('created missing directory {}'.format(jbot_dir))

        fs_path = os.path.join(jbot_dir, site_id)
        if not os.path.exists(fs_path):
            os.mkdir(fs_path)
            logger.info('created missing directory {}'.format(fs_path))
        return fs_path

    def getFileFromDirectory(self, directory, filename):  # noqa
        fi = directory[filename]
        path = self._get_fs_path()
        filepath = os.path.join(path, filename)
        if fi.__class__.__name__ == "FilesystemFile":
            last_modified = fi.lastModifiedTimestamp
        else:
            last_modified = fi._p_mtime
        if not os.path.exists(filepath) or \
                DateTime(last_modified) > DateTime(os.stat(filepath).st_mtime):
            tmpfi = open(filepath, 'wb')
            if fi.__class__.__name__ == "FilesystemFile":
                data = IRawReadFile(fi).read()
            else:
                data = str(fi.data)
            tmpfi.write(data)
            tmpfi.close()
        return filepath

    def get(self, filename, template):
        # get path to file for template
        if self.themeDirectory and filename in self.themeDirectory:
            directory = self.themeDirectory
        return self.getFileFromDirectory(directory, filename)


class TemplateManager(object):
    interface.implements(interfaces.ITemplateManager)

    def __init__(self, name):
        self.name = name
        self._req = None
        self.syspaths = tuple(sys.path)

    @property
    def customized_filenames(self):
        key = '%s.%s' % (REQ_CACHE_KEY, 'customized_filenames')
        if key not in self.req.environ:
            storage = self.storage
            if storage:
                files = []
                if storage.themeDirectory:
                    files.extend(storage.themeDirectory.listDirectory())
                self.req.environ[key] = files
            else:
                self.req.environ[key] = []
        return self.req.environ[key]

    @property
    def req(self):
        if self._req is None:
            self._req = getRequest()
        return self._req

    @property
    def paths(self):
        key = '%s.%s' % (REQ_CACHE_KEY, 'paths')
        if key not in self.req.environ:
            self.req.environ[key] = {}
        return self.req.environ[key]

    @property
    def templates(self):
        key = '%s.%s' % (REQ_CACHE_KEY, 'templates')
        if key not in self.req.environ:
            self.req.environ[key] = {}
        return self.req.environ[key]

    @property
    def storage(self):
        if REQ_CACHE_KEY not in self.req.environ:
            try:
                self.req.environ[REQ_CACHE_KEY] = Storage()
            except ComponentLookupError:
                self.req.environ[REQ_CACHE_KEY] = None
        return self.req.environ[REQ_CACHE_KEY]

    def registerTemplate(self, template, token):  # noqa
        """
        wrap this method so we can log errors easily
        """
        try:
            return self._registerTemplate(template, token)
        except:
            if 'jbot.error' not in self.req.environ:
                # only log one error per request
                self.req.environ['jbot.error'] = True
                logger.error(
                    'collective.jbot error: %s' % traceback.format_exc())

    def _registerTemplate(self, template, token):  # noqa
        """
        Return True if there has been a change to the override.
        Due to the nature of this implementation, a multi-site deployment
        could cause the template setting of resources to be re-set
        often. This is probably not optimal. And with chamelean I wonder if
        the compiling is re-done each time also... Eek
        """
        # assert that the template is not already registered
        filename = self.templates.get(token)
        if filename is IGNORE:
            return

        # if the template filename matches an override, we're done
        paths = self.paths
        if paths.get(filename) == template.filename and \
                os.path.exists(filename):
            return

        # verify that override has not been unregistered
        if filename is not None and filename not in paths:
            template.filename = template._filename
            del self.templates[token]

        customized = False
        if hasattr(template, '__filename'):
            # already customized by us
            path = find_package(self.syspaths, template._filename)
            customized = True
        else:
            # check if an overridable resource
            path = find_package(self.syspaths, template.filename)
            if path is None:
                # permanently ignore template
                self.templates[token] = IGNORE
                return

        filename = path.replace(os.path.sep, '.')
        # check again if filename in list of cached paths. Might be diff
        # object but same filename, we return a customized version here
        # because one request object corresponds to one site customization
        if filename in self.paths:
            if filename != template.filename:
                if not hasattr(template, '__filename'):
                    template.__filename = template.filename
                template.filename = self.paths[filename]
                return True

        if not self.storage:
            if customized:
                # revert now...
                template.filename = template.__filename
                del template.__filename
                return True
        else:
            if filename in self.customized_filenames:
                filepath = self.storage.get(filename, template)
                if filepath == template.filename:
                    # already customized with correct path, ignore
                    return

                self.paths[filename] = filepath

                # save original filename
                if not hasattr(template, '__filename'):
                    template.__filename = template.filename

                # save template, registry and assign path
                template.filename = filepath
                self.templates[token] = filename

                return True
            else:
                if customized:
                    template.filename = template.__filename
                    del template.__filename
                    return True



from castle.cms.behaviors.location import ILocation
from castle.cms.tiles.base import BaseTile
from castle.cms.utils import parse_query_from_data
from castle.cms.widgets import MapMarkersFieldWidget
from castle.cms.widgets import MapPointFieldWidget
from castle.cms.widgets import QueryFieldWidget
from castle.cms.widgets import RelatedItemsFieldWidget
from castle.cms.widgets import UseQueryWidget
from plone import api
from plone.autoform import directives as form
from plone.registry.interfaces import IRegistry
from plone.supermodel import model
from plone.tiles.interfaces import IPersistentTile
from zope import schema
from zope.component import getUtility
from zope.interface import implements
from zope.interface import Invalid
from zope.interface import invariant

import json


ITEM_TEMPLATE = """
<h3><a href="%(url)s">%(title)s</a></h3>
<p>%(description)s</p>
"""


class MapTile(BaseTile):
    implements(IPersistentTile)

    def render(self):
        return self.index()

    @property
    def query(self):
        parsed = parse_query_from_data(self.data, self.context)
        parsed['sort_limit'] = self.data.get('limit', 20) or 20
        return parsed

    def json_data(self):

        registry = getUtility(IRegistry)
        typesUseViewActionInListings = frozenset(
            registry.get('plone.types_use_view_action_in_listings', []))

        try:
            markers = json.loads(self.data.get('custom_markers', '[]') or '[]')
        except:
            markers = []
        brains = []
        catalog = api.portal.get_tool('portal_catalog')
        if self.data.get('use_query') in ('True', True, 'true'):
            brains = catalog(**self.query)[:self.data.get('limit', 20) or 20]
        else:
            brains = catalog(UID=self.data.get('content', []) or [])
        for brain in brains:
            ob = brain.getObject()
            data = ILocation(ob, None)
            if data is None:
                continue
            try:
                coords = json.loads(data.coordinates)
            except:
                continue
            if not coords:
                continue

            if type(coords) == dict:
                coords = [coords]

            title = ob.Title()
            url = ob.absolute_url()
            if ob.portal_type in typesUseViewActionInListings:
                url += '/view'
            html = ITEM_TEMPLATE % {
                'title': title,
                'description': ob.Description(),
                'url': url
            }

            for coord in coords:
                coord = coord.copy()
                coord['popup'] = html
                markers.append(coord)
        data = {
            'initialZoom': self.data.get('initialZoom', 11) or 11,
            'height': self.data.get('height', 200) or 200,
            'center': self.data.get('center'),
            'markers': json.dumps(markers)
        }
        return json.dumps(data)


def validate_center(center):
    try:
        center = json.loads(center)
        if len(center) == 0:
            return False
    except:
        pass
    return True


class IMapTileSchema(model.Schema):

    form.widget(use_query=UseQueryWidget)
    use_query = schema.Bool(
        title=u'Use dynamic query',
        description=u'For map content.',
        default=False)

    form.widget(content=RelatedItemsFieldWidget)
    content = schema.List(
        title=u"Items",
        description=u"Selected items to show on map",
        required=False,
        default=[],
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.Catalog'
        )
    )

    form.widget(query=QueryFieldWidget)
    query = schema.List(
        title=u'Search terms',
        description=u"Define the search terms for the items you want "
                    u"dynamically include in this map",
        value_type=schema.Dict(value_type=schema.Field(),
                               key_type=schema.TextLine()),
        required=False
    )

    sort_on = schema.TextLine(
        title=u'Sort on',
        description=u"Sort on this index",
        required=False,
    )

    sort_reversed = schema.Bool(
        title=u'Reversed order',
        description=u'Sort the results in reversed order',
        required=False,
    )

    limit = schema.Int(
        title=u'Limit',
        description=u'Limit Search Results',
        required=False,
        default=20,
        min=1,
    )

    form.widget(custom_markers=MapMarkersFieldWidget)
    custom_markers = schema.Text(
        title=u'Custom Markers',
        default=u'[]',
        required=False
    )

    form.widget(center=MapPointFieldWidget)
    center = schema.Text(
        title=u'Center point',
        default=u'{}'
    )

    @invariant
    def validate_center(data):
        try:
            center = json.loads(data.center)
        except:
            return
        if len(center) == 0:
            raise Invalid('Must provide a center point')

    height = schema.Int(
        title=u"Height",
        default=200
    )

    initialZoom = schema.Int(
        title=u"Initial Zoom",
        default=11
    )



from castle.cms import registration
from castle.cms.utils import get_email_from_address
from castle.cms.utils import send_email
from castle.cms.utils import verify_recaptcha
from castle.cms.widgets import ReCaptchaFieldWidget
from plone.app.users.browser.register import RegistrationForm as BaseRegistrationForm
from plone.app.users.schema import checkEmailAddress
from plone.autoform.form import AutoExtensibleForm
from plone.registry.interfaces import IRegistry
from plone.z3cform.fieldsets import utils as z3cform_utils
from plone.z3cform.fieldsets.utils import move
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import button
from z3c.form import field
from z3c.form import form
from z3c.form import interfaces
from z3c.form.action import ActionErrorOccurred
from z3c.form.interfaces import WidgetActionExecutionError
from zope import schema
from zope.component import getUtility
from zope.component import queryUtility
from zope.event import notify
from zope.interface import Interface
from zope.interface import Invalid


class IEmailConfirmation(Interface):
    email = schema.ASCIILine(
        title=_(u'label_email', default=u'E-mail'),
        description=u'',
        required=True,
        constraint=checkEmailAddress)

    captcha = schema.TextLine(
        title=u"Captcha",
        required=False)


class EmailConfirmation(AutoExtensibleForm, form.Form):
    label = u"Confirm your email address"
    description = (u"Before you can begin the registration process, you need to "
                   u"verify your email address.")
    formErrorsMessage = _('There were errors.')
    ignoreContext = True
    schema = IEmailConfirmation
    enableCSRFProtection = True
    template = ViewPageTemplateFile('templates/confirm-email.pt')
    sent = False

    def __init__(self, context, request):
        super(EmailConfirmation, self).__init__(context, request)
        registry = queryUtility(IRegistry)
        self.has_captcha = registry.get('castle.recaptcha_private_key') is not None
        portal_membership = getToolByName(self.context, 'portal_membership')
        self.isAnon = portal_membership.isAnonymousUser()

    def send_mail(self, email, item):
        url = '%s/@@register?confirmed_email=%s&confirmed_code=%s' % (
            self.context.absolute_url(), email, item['code'])
        text = """
Copy and paste this url into your web browser to confirm your address: %s
""" % url
        html = """
<p>You have requested registration, please
<a href="%s">confirm your email address by clicking on this link</a>.
</p>
<p>
If that does not work, copy and paste this urls into your web browser: %s
</p>""" % (url, url)
        send_email(
            [email], "Email Confirmation",
            html=html, text=text)

    def updateFields(self):
        super(EmailConfirmation, self).updateFields()
        if self.has_captcha and self.isAnon:
            self.fields['captcha'].widgetFactory = ReCaptchaFieldWidget
        else:
            self.fields['captcha'].mode = interfaces.HIDDEN_MODE

        move(self, 'email', before='*')

    @button.buttonAndHandler(
        _(u'label_verify', default=u'Verify'), name='verify'
    )
    def action_verify(self, action):
        data, errors = self.extractData()
        registry = queryUtility(IRegistry)
        has_captcha = registry.get('castle.recaptcha_private_key') is not None
        if has_captcha:
            if not verify_recaptcha(self.request):
                notify(
                    ActionErrorOccurred(
                        action,
                        WidgetActionExecutionError('captcha', Invalid('Invalid Recaptcha'))))
                return

        if not errors:
            storage = registration.RegistrationStorage(self.context)
            item = storage.add(data['email'])
            self.send_mail(data['email'], item)
            self.sent = True
            IStatusMessage(self.request).addStatusMessage(
                'Verification email has been sent to your email.', type='info')


class IHiddenVerifiedEmail(Interface):

    confirmed_email = schema.TextLine()
    confirmed_code = schema.TextLine()


class RegistrationForm(BaseRegistrationForm):

    def get_confirmed_email(self):
        req = self.request
        return req.form.get('confirmed_email', req.form.get('form.widgets.confirmed_email', ''))

    def get_confirmed_code(self):
        req = self.request
        return req.form.get(
            'confirmed_code', req.form.get('form.widgets.confirmed_code', ''))

    def verify(self):
        email = self.get_confirmed_email()
        code = self.get_confirmed_code()
        if not email or not code:
            return False
        storage = registration.RegistrationStorage(self.context)
        entry = storage.get(email)
        if entry is None:
            return False
        if entry['code'] == code:
            return True
        return False

    def updateWidgets(self):
        if self.showForm:
            super(RegistrationForm, self).updateWidgets()
        else:
            form.Form.updateWidgets(self)
        self.widgets['confirmed_email'].value = self.get_confirmed_email()
        self.widgets['confirmed_code'].value = self.get_confirmed_code()

    def validate_registration(self, action, data):
        super(RegistrationForm, self).validate_registration(action, data)

        if 'email' in data and data['email'].lower() != self.get_confirmed_email().lower():
            err_str = u'Email address you have entered does not match email used in verification'
            notify(
                ActionErrorOccurred(
                    action, WidgetActionExecutionError('email', Invalid(err_str))
                )
            )
        del data['confirmed_email']
        del data['confirmed_code']

    def handle_join_success(self, data):
        email = self.get_confirmed_email()
        storage = registration.RegistrationStorage(self.context)
        storage.remove(email)
        registry = getUtility(IRegistry)
        try:
            review = registry['plone.review_registrations']
        except KeyError:
            review = False
            pass
        if review:
            storage = registration.RegistrationReviewStorage(self.context)
            storage.add(email, data)
            self.send_email_to_admin_to_review(email)
            self.request.response.redirect('%s/@@under-review?email=%s' % (
                self.context.absolute_url(), email))
        else:
            return super(RegistrationForm, self).handle_join_success(data)

    def send_email_to_admin_to_review(self, email):
        url = '%s/@@review-registration-requests' % (
            self.context.absolute_url())
        text = """
Hi,

A new user with the email %(email)s has signed up.

You can review the request at %(url)s
""" % {
            'url': url,
            'email': email
        }
        html = """
<p>Hi,</p>

A new user with the email %(email)s has signed up.

Please <a href="%s">review the request</a>

</p>""" % {
            'url': url,
            'email': email
        }
        send_email(
            [get_email_from_address()], "User registration needs review",
            html=html, text=text)

    def updateFields(self):
        super(RegistrationForm, self).updateFields()
        if self.showForm:
            z3cform_utils.add(self, IHiddenVerifiedEmail, prefix="")
        else:
            self.fields = field.Fields(IHiddenVerifiedEmail, prefix="")
        self.fields['confirmed_email'].mode = interfaces.HIDDEN_MODE
        self.fields['confirmed_code'].mode = interfaces.HIDDEN_MODE

    def __call__(self):
        if not self.verify():
            return self.request.response.redirect('%s/@@register-confirm-email' % (
                self.context.absolute_url()))

        return super(RegistrationForm, self).__call__()


class ReviewRequests(BrowserView):

    def enabled(self):
        registry = getUtility(IRegistry)
        try:
            return registry['plone.review_registrations']
        except KeyError:
            return False

    def send_approve_mail(self, email, data):
        data = data.copy()
        text = """
Hello %(fullname)s,

The user with username "%(username)s" has been approved.

You can visit the site at: %(url)s
""" % data
        html = """
<p>Hello %(fullname)s,</p>

<p>The user with username "%(username)s" has been approved.</p>

<p>You can visit the site at: <a href="%(url)s">%(url)s</a>
</p>""" % data
        send_email(
            [email], "User approved",
            html=html, text=text)

    def __call__(self):
        storage = registration.RegistrationReviewStorage(self.context)
        if self.request.REQUEST_METHOD == 'POST':
            email = self.request.form.get('email')
            if self.request.form.get('approve'):
                data = storage.get(email).copy()
                data.pop('code')
                data.pop('created')
                reg_form = BaseRegistrationForm(self.context, self.request)
                reg_form.updateFields()
                reg_form.updateWidgets()
                reg_form.handle_join_success(data)
                if data.get('password'):
                    # won't get an email so sent them out something about getting approved
                    self.send_approve_mail(email, data)
                storage.remove(email)
            elif self.request.form.get('deny'):
                storage.remove(email)
            elif self.request.form.get('enable'):
                getUtility(IRegistry)['plone.review_registrations'] = True
            elif self.request.form.get('disable'):
                getUtility(IRegistry)['plone.review_registrations'] = False
        self.storage = storage
        self.data = storage._data
        return self.index()


class UnderReview(BrowserView):

    def __call__(self):
        storage = registration.RegistrationReviewStorage(self.context)
        self.data = storage.get(self.request.form.get('email'))
        return self.index()
